/********************************************************************************************
 *
 * Copyright 2015  Frank Gollnik
 * 
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.console.de;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.regex.Pattern;


/**
 * Die Klasse XKonsole implementiert die Schnittstelle IKonsole, dient
 * also der Ein- und Ausgabe von Daten der Typen String (=Zeichenkette),
 * int (=Ganzzahl) und float (=Fließkommazahl).
 * Die Klasse ist allein für Lehrzwecke konzipiert. Ihre Verwendung
 * erfolgt ganz analog zu den in org.jabotics.robot.de definierten
 * Schnittstellen für die Robotersteuerung. Anhand dieser Klasse lassen
 * sich elementare Java-Kenntnisse erarbeiten, bevor man auf die etwas
 * komplexeren Steuerungsklassen für den Roboter übergeht.
 * 
 * @author Frank Gollnik
 * @since   08.2012
 * @version 09.2015
 */
public class XKonsole implements IKonsole
{
    // Instanzvariablen
    private String mMusterGanzzahl;
    private String mMusterFliesskommazahl;
    private BufferedReader mReader;

    /**
     * Öffentlicher Standard-Konstruktor für Objekte der Klasse XKonsole.
     */
    public XKonsole()
    {
        // Instanzvariable initialisieren
        // Zu den Mustern siehe API-Doku von java.util.regex.Pattern
        mMusterGanzzahl = "^[+-]?\\d+$"; // \d entspricht [0-9]
        mMusterFliesskommazahl = "^[-+]?[0-9]+[.]?[0-9]*([eE][-+]?[0-9]+)?$";
        mReader = new BufferedReader(new InputStreamReader(System.in));
    }

    public String leseText()
    {
        String ergebnis = "";
        try
        {
            ergebnis = mReader.readLine();
            while (ergebnis.isEmpty())
            {
                System.out.println("Die eingegebene Zeichenkette ist leer. Eingabe bitte wiederholen!");
                ergebnis = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return ergebnis;
    }

    public String leseText(String abfragetext)
    {
        String ergebnis = "";
        try
        {
            System.out.println(abfragetext);
            ergebnis = mReader.readLine();
            while (ergebnis.isEmpty())
            {
                System.out.println("Die eingegebene Zeichenkette ist leer. Eingabe bitte wiederholen!");
                ergebnis = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return ergebnis;
    }

    public int leseGanzzahl()
    {
        String Zeile = "";
        try
        {
            Zeile = mReader.readLine();
            while (!Pattern.matches(mMusterGanzzahl, Zeile))
            {
                System.out.println("Die Eingabe besitzt nicht das Format einer Ganzzahl.");
                System.out.println("Eingabe bitte wiederholen!");
                Zeile = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return Integer.parseInt(Zeile);
    }

    public int leseGanzzahl(String abfragetext)
    {
        String Zeile = "";
        try
        {
            System.out.println(abfragetext);
            Zeile = mReader.readLine();
            while (!Pattern.matches(mMusterGanzzahl, Zeile))
            {
                System.out.println("Die Eingabe besitzt nicht das Format einer Ganzzahl.");
                System.out.println("Eingabe bitte wiederholen!");
                Zeile = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return Integer.parseInt(Zeile);
    }

    public float leseZahl()
    {
        String Zeile = "";
        try
        {
            Zeile = mReader.readLine();
            while (!Pattern.matches(mMusterFliesskommazahl, Zeile))
            {
                System.out.println("Die Eingabe besitzt nicht das Format einer Fließkommazahl.");
                System.out.println("Eingabe bitte wiederholen!");
                Zeile = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return Float.parseFloat(Zeile);
    }

    public float leseZahl(String abfragetext)
    {
        String Zeile = "";
        try
        {
            System.out.println(abfragetext);
            Zeile = mReader.readLine();
            while (!Pattern.matches(mMusterFliesskommazahl, Zeile))
            {
                System.out.println("Die Eingabe besitzt nicht das Format einer Fließkommazahl.");
                System.out.println("Eingabe bitte wiederholen!");
                Zeile = mReader.readLine();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return Float.parseFloat(Zeile);
    }

    public void schreibe(String text)
    {
        System.out.print(text);
    }

    public void schreibe(int ganzzahl)
    {
        System.out.print(ganzzahl);
    }

    public void schreibe(float fliesskommazahl)
    {
        System.out.print(fliesskommazahl);
    }

    public void schreibeZeile(String text)
    {
        System.out.println(text);
    }

    public void schreibeZeile(int ganzzahl)
    {
        System.out.println(ganzzahl);
    }

    public void schreibeZeile(float fliesskommazahl)
    {
        System.out.println(fliesskommazahl);
    }
}
