/********************************************************************************************
 *
 * Copyright 2015  Frank Gollnik
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.console.en;


/**
 * The interface IConsole defines methods for the input and output of data
 * of type String (text), int (integral number) und float (floating point number)
 * in the context of a simple console application.
 * 
 * @author Frank Gollnik
 * @since   09.2015
 * @version 09.2015
 */

public interface IConsole
{
    /**
     * Input of a line of text.
     * The user will be asked to repeat the input if the line is empty.
     * 
     * @return        the entered line of text
     */
    public String readText();

    /**
     * Input of a line of text.
     * The user will be asked to repeat the input if the line is empty.
     * 
     * @param  prompt   the prompt that is displayed to the user prior to the input
     * @return          the entered line of text
     */
    public String readText(String prompt);

    /**
     * Input of an integral number.
     * The user will be asked to repeat the input if it is in the wrong format.
     * 
     * @return          the entered integer
     */
    public int readInteger();

    /**
     * Input of an integral number.
     * The user will be asked to repeat the input if it is in the wrong format.
     * 
     * @param  prompt   the prompt that is displayed to the user prior to the input
     * @return          the entered integer
     */
    public int readInteger(String prompt);

    /**
     * Input of a floating-point number.
     * The user will be asked to repeat the input if it is in the wrong format.
     * 
     * @return          the entered floating-point number
     */
    public float readNumber();

    /**
     * Input of a floating-point number.
     * The user will be asked to repeat the input if it is in the wrong format.
     * 
     * @param  prompt   the prompt that is displayed to the user prior to the input
     * @return          the entered floating-point number
     */
    public float readNumber(String prompt);

    /**
     * Output of text without line break.
     * 
     * @param  text     the text that is to be displayed on the screen
     */
    public void write(String text);

    /**
     * Output of an integral number without line break.
     * 
     * @param  integer  the integer that is to be displayed on the screen
     */
    public void write(int integer);

    /**
     * Output of a floating-point number without line break.
     * 
     * @param  number   the floating-point number that is to be displayed on the screen
     */
    public void write(float number);

    /**
     * Output of text with line break.
     * 
     * @param  text     the text that is to be displayed on the screen
     */
    public void writeLine(String text);

    /**
     * Output of an integral number with line break.
     * 
     * @param  integer  the integer that is to be displayed on the screen
     */
    public void writeLine(int integer);

    /**
     * Output of a floating-point number with line break.
     * 
     * @param  number   the floating-point number that is to be displayed on the screen
     */
    public void writeLine(float number);
}
