/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.adapter;

import org.jabotics.robot.defs.*;
import org.jabotics.robot.compile.*;

/**
 * Depending on the settings defined in org.jabotics.robot.compile, this
 * class will link to the appropriate implementation of IAdapterFactory.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class XAdapter
{
    static final private IAdapterFactory sFactory;
    
    static
    {
        // The initialization of sFactory is not generally done by a call to an appropriate factory provider method
        // (like:  if (CS.NXT) sFactory = NXTAdapter.getFactory();) because in this case all factory (provider) classes
        // would be parsed when compiling XAdapter. In general not all necessary libraries will be accessible
        // when performing the compilation for a certain target product. E.g. when compiling for the NXT robot
        // with its reduced functionality and java class libraries provided by leJOS, the package org.jabotics.robot.sim
        // (containing SIMAdapter) cannot be processed. Therefore this package needs to be "hidden" when
        // compiling XAdapter. The following code adresses this issue but did not work either, because leJOS
        // neither supports the method Class.newInstance() (problem at compile time) nor Class.forName()
        // (always throws ClassNotFoundException and therefore problem at runtime).
        //
        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Use Class.forName(<qualified class name>) and Class.newInstance() to create an instance of the factory class.
        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /*
        IAdapterFactory factory = null;
        try
        {
            Class t;
            if (CS.NXT) t = Class.forName("org.jabotics.robot.nxt.NXTAdapter");
            if (CS.SIM) t = Class.forName("org.jabotics.robot.sim.SIMAdapter");

            if (null != t) 
            {
                Object obj = t.newInstance();
                if (obj instanceof IAdapterFactory) factory = (IAdapterFactory) obj;
            }
        }
        catch (Exception e) {}
        sFactory = factory;
        */
        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //
        // The next ideas was to simply make a call to Class.forName(<class name>) which triggers the initialization
        // of the corresponding class. As part of this class initialization routine the adapter factory will be passed
        // to a helper class XAdapterFactoryHandle that can be accaccessed by XAdapter in the next step (kind of
        // "indirect initialization").
        // As already mentioned, this approach does not work for leJOS since Class.forName always throws an exception.
        // Therefore a combined approach has been chosen:
        // - hide package org.jabotics.robot.sim (and others) by means of Class.forName
        // - do not hide org.jabotics.robot.nxt but provide class library for package lejos in classpath during the build
        //   of products other than nxt. The corresponding file Lib/lejos-package.jar has been created from the original
        //   lejos-jar-file classes.jar by simply removing the packages java and javax.
        //
        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Current implementation:
        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        IAdapterFactory factory = null;
        try
        {
            if (CS.NXT) factory = org.jabotics.robot.nxt.NXTAdapter.getAdapterFactory(); // => class org.jabotics.robot.nxt.NXTAdapter will be parsed in any case
            if (CS.EV3)
            {
                Class.forName("org.jabotics.robot.ev3.EV3Adapter"); // Initialization routine of EV3Adapter will pass factory to helper class XAdapterFactoryHandle.
                factory = XAdapterFactoryHandle.get();  // XAdapterFactoryHandle should now be initialized
            }
            if (CS.SIM)
            {
                Class.forName("org.jabotics.robot.sim.SIMAdapter"); // Initialization routine of SIMAdapter will pass factory to helper class XAdapterFactoryHandle.
                factory = XAdapterFactoryHandle.get();  // XAdapterFactoryHandle should now be initialized
            }
        }
        catch (Exception e) {}
        sFactory = factory;
    }

    static public IAdapterFactory getFactory()
    {
        return sFactory;
    }
}
