/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.bluej; 

import bluej.extensions.*;
import bluej.extensions.event.*;
import bluej.extensions.editor.*;

//import java.util.Locale;
import java.net.URL;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Map;
import java.util.ArrayList;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;

/**
 * Stellt eine BlueJ-Erweiterung zur Arbeit mit Lego-Robotern zur Verfügung
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class XBlueJExtension extends Extension
{
    static final String VERSION_NUMBER =    "1.0.3";
    static final String COPYRIGHT =         "2012-2016";

    static final String NXT_BROWSER_ACTION =          "NXT_BROWSER_ACTION";
    static final String NXT_CONSOLE_ACTION =          "NXT_CONSOLE_ACTION";
    static final String NXT_FLASH_ACTION  =           "NXT_FLASH_ACTION";
    static final String NXT_COMPILE_ACTION =          "NXT_COMPILE_ACTION";
    static final String NXT_COMPILEPACKAGE_ACTION =   "NXT_COMPILEPACKAGE_ACTION";
    static final String NXT_UPLOAD_ACTION =           "NXT_UPLOAD_ACTION";
    static final String EV3_CONTROL_ACTION =          "EV3_CONTROL_ACTION";
    static final String EV3_SELECT_ROBOT_ACTION =     "EV3_SELECT_ROBOT_ACTION";
    static final String EV3_SDCARD_ACTION  =          "EV3_SDCARD_ACTION";
    static final String EV3_LIBS_UPLOAD_ACTION =      "EV3_LIBS_UPLOAD_ACTION";
    static final String EV3_COMPILE_ACTION =          "EV3_COMPILE_ACTION";
    static final String EV3_COMPILEPACKAGE_ACTION =   "EV3_COMPILEPACKAGE_ACTION";
    static final String EV3_UPLOAD_ACTION =           "EV3_UPLOAD_ACTION";
    static final String SIMULATE_ACTION =             "SIMULATE_ACTION";
    static final String[] VM_NAMES_WIN =              {"jdk1.9","jdk1.8","jdk1.7","jdk1.6"};
    static final String[] VM_NAMES_LINUX =            {"java-9-oracle","jdk-9-oracle","java-9-openjdk",
                                                       "java-8-oracle","jdk-8-oracle","java-8-openjdk",
                                                       "java-7-oracle","jdk-7-oracle","java-7-sun","java-7-openjdk",
                                                       "java-6-oracle","java-6-sun","java-6-openjdk"};

    // Assign keys, first
    protected String MSG_ROBOT =                  "robot.prompt";
    protected String MSG_ROBOT_EXT =              "robot.ext.title";
    protected String MSG_ROBOT_EXT_INFO =         "robot.ext.info";

    protected String MSG_INFO =                   "info.prompt";
    protected String MSG_VERSION =                "info.version";
    protected String MSG_AUTHOR =                 "info.author";

    protected String MSG_WARNING =                "warn.prompt";
    protected String MSG_ERROR =                  "err.prompt";
    protected String MSG_URL_ERROR =              "err.url";

    protected String MSG_ENV_VAR =                "env.prompt";
    protected String MSG_ENV_VAR_OK =             "env.valid";
    protected String MSG_ENV_VAR_NOT_OK =         "env.invalid";
    protected String MSG_ENV_VAR_NOT_DEFINED =    "env.undefined";
    protected String MSG_DIR_FOUND =              "dir.yes";
    protected String MSG_DIR_NOT_FOUND =          "dir.no";
    protected String MSG_FILE_FOUND =             "file.yes";
    protected String MSG_FILE_NOT_FOUND =         "file.no";
    protected String MSG_NO_JAR_FILE_IN_DIR =     "file.jar.no";
    protected String MSG_JVM_FOUND =              "jvm.yes";
    protected String MSG_JVM_NOT_FOUND =          "jvm.no";
    protected String MSG_NXJ_FOUND =              "nxj.yes";
    protected String MSG_NXJ_NOT_FOUND =          "nxj.no";
    protected String MSG_EV3_FOUND =              "ev3.yes";
    protected String MSG_EV3_NOT_FOUND =          "ev3.no";

    protected String MSG_SHOW_DETAILS =           "details.show";
    protected String MSG_INIT_DETAILS =           "details.init";

    protected String MSG_COMPILING_PACKAGE =      "comp.package";
    protected String MSG_COMPILING =              "comp.prompt";
    protected String MSG_COMPILATION_RUNNING =    "comp.running";
    protected String MSG_COMPILATION_COMPLETED =  "comp.completed";
    protected String MSG_COMPILATION_FAILED =     "comp.failed";

    protected String MSG_JAR =                    "jar.prompt";
    protected String MSG_JAR_COMPLETED =          "jar.completed";
    protected String MSG_JAR_FAILED =             "jar.failed";

    protected String MSG_SIMULATION =             "sim.title";
    protected String MSG_SIM_INIT =               "sim.init";
    protected String MSG_SIM_READY =              "sim.ready.yes";
    protected String MSG_SIM_NOT_READY =          "sim.ready.no";
    protected String MSG_SIM_START =              "sim.prompt";
    protected String MSG_SIM_STARTED =            "sim.started";
    protected String MSG_SIM_RUNNING =            "sim.running";

    protected String MSG_NXT =                    "nxt.prompt";
    protected String MSG_NXT_INIT =               "nxt.init";
    protected String MSG_NXT_READY =              "nxt.ready.yes";
    protected String MSG_NXT_NOT_READY =          "nxt.ready.no";
    protected String MSG_NXT_BROWSER =            "nxt.browser.prompt";
    protected String MSG_NXT_BROWSER_STARTED =    "nxt.browser.started";
    protected String MSG_NXT_BROWSER_RUNNING =    "nxt.browser.running";
    protected String MSG_NXT_CONSOLE =            "nxt.console.prompt";
    protected String MSG_NXT_CONSOLE_STARTED =    "nxt.console.started";
    protected String MSG_NXT_CONSOLE_RUNNING =    "nxt.console.running";
    protected String MSG_NXT_FLASHTOOL =          "nxt.flash.prompt";
    protected String MSG_NXT_FLASHTOOL_STARTED =  "nxt.flash.started";
    protected String MSG_NXT_FLASHTOOL_RUNNING =  "nxt.flash.running";
    protected String MSG_NXT_COMPILEPACKAGE =     "nxt.comp.package";
    protected String MSG_NXT_COMPILE =            "nxt.comp.prompt";
    protected String MSG_NXT_UPLOAD =             "nxt.upload.prompt";
    protected String MSG_NXT_UPLOAD_RUNNING =     "nxt.upload.running";

    protected String MSG_EV3 =                    "ev3.prompt";
    protected String MSG_EV3_INIT =               "ev3.init";
    protected String MSG_EV3_READY =              "ev3.ready.yes";
    protected String MSG_EV3_NOT_READY =          "ev3.ready.no";
    protected String MSG_EV3_CONTROL =            "ev3.control.prompt";
    protected String MSG_EV3_CONTROL_STARTED =    "ev3.control.started";
    protected String MSG_EV3_CONTROL_RUNNING =    "ev3.control.running";
    protected String MSG_EV3_ROBOT =              "ev3.robot.prompt";
    protected String MSG_EV3_ROBOT_ADDRESS =      "ev3.robot.address";
    protected String MSG_EV3_SDCARD =             "ev3.sdcard.prompt";
    protected String MSG_EV3_SDCARD_STARTED =     "ev3.sdcard.started";
    protected String MSG_EV3_SDCARD_RUNNING =     "ev3.sdcard.running";
    protected String MSG_EV3_LIBS =               "ev3.libs.prompt";
    protected String MSG_EV3_LIBS_RUNNING =       "ev3.libs.running";
    protected String MSG_EV3_LIBS_COMPLETED =     "ev3.libs.completed";
    protected String MSG_EV3_LIBS_FAILED =        "ev3.libs.failed";
    protected String MSG_EV3_COMPILEPACKAGE =     "ev3.comp.package";
    protected String MSG_EV3_COMPILE =            "ev3.comp.prompt";
    protected String MSG_EV3_UPLOAD =             "ev3.upload.prompt";
    protected String MSG_EV3_UPLOAD_RUNNING =     "ev3.upload.running";
    protected String MSG_EV3_UPLOAD_COMPLETED =   "ev3.upload.completed";
    protected String MSG_EV3_UPLOAD_FAILED =      "ev3.upload.failed";

    protected BlueJ mBlueJ;
    protected String mLanguage;
    protected boolean mReadyNXT;
    protected boolean mReadyEV3;
    protected boolean mReadySIM;
    protected boolean mWindows;
    protected String mInitDetails;
    protected String mInfo;
    protected String mClasspathUserlibBlueJ;
    protected String mClasspathUserlibNXJ;
    protected String mClasspathEV3;
    protected ArrayList<File> mFilesUserlibEV3 = new ArrayList<File>();
    protected ArrayList<String> mBrowserCommandListNXT = new ArrayList<String>();
    protected ArrayList<String> mConsoleCommandListNXT = new ArrayList<String>();
    protected ArrayList<String> mFlashCommandListNXT = new ArrayList<String>();
    protected ArrayList<String> mCompileCommandListNXT = new ArrayList<String>();
    protected ArrayList<String> mLinkAndUploadCommandListNXT = new ArrayList<String>();
    protected ArrayList<String> mControlCommandListEV3 = new ArrayList<String>();
    protected ArrayList<String> mSDCardCommandListEV3 = new ArrayList<String>();
    protected ArrayList<String> mLibUploadCommandListEV3 = new ArrayList<String>();
    protected ArrayList<String> mCompileCommandListEV3 = new ArrayList<String>();
    protected ArrayList<String> mJarCommandListEV3 = new ArrayList<String>();
    protected ArrayList<String> mUploadCommandListEV3 = new ArrayList<String>();
    protected ArrayList<String> mSimulateCommandList = new ArrayList<String>();
    protected ArrayList<String> mCommandList = new ArrayList<String>();
    protected ProcessBuilder mPB;
    
    public void startup (BlueJ bluej)
    {
        mBlueJ = bluej;

        startup(); // alle sonstigen Vorbereitungen (vor Setzen von MenuGenerator und PreferenceGenerator zu erledigen)
        
        // Register a generator for menu items
        bluej.setMenuGenerator(new XMenuGenerator());

        // Register a "preferences" panel generator
        bluej.setPreferenceGenerator(new XPreferences(bluej));
    }

    private void initMessages()
    {
        // do not allow the initialization of the extension to crash due to a problem with the internationalization
        try
        {
            Locale currentLocale = new Locale(mLanguage);

            ResourceBundle messages = ResourceBundle.getBundle("resources/properties/jabotics-bluej-messages", currentLocale);
    
            // Replace key by translation
            MSG_ROBOT =                  messages.getString(MSG_ROBOT);
            MSG_ROBOT_EXT =              messages.getString(MSG_ROBOT_EXT);
            MSG_ROBOT_EXT_INFO =         messages.getString(MSG_ROBOT_EXT_INFO);

            MSG_VERSION =                messages.getString(MSG_VERSION);
            MSG_AUTHOR =                 messages.getString(MSG_AUTHOR);

            MSG_INFO =                   messages.getString(MSG_INFO);
            MSG_WARNING =                messages.getString(MSG_WARNING);
            MSG_ERROR =                  messages.getString(MSG_ERROR);
            MSG_URL_ERROR =              messages.getString(MSG_URL_ERROR);

            MSG_ENV_VAR =                messages.getString(MSG_ENV_VAR);
            MSG_ENV_VAR_OK =             messages.getString(MSG_ENV_VAR_OK);
            MSG_ENV_VAR_NOT_OK =         messages.getString(MSG_ENV_VAR_NOT_OK);
            MSG_ENV_VAR_NOT_DEFINED =    messages.getString(MSG_ENV_VAR_NOT_DEFINED);
            MSG_DIR_FOUND =              messages.getString(MSG_DIR_FOUND);
            MSG_DIR_NOT_FOUND =          messages.getString(MSG_DIR_NOT_FOUND);
            MSG_FILE_FOUND =             messages.getString(MSG_FILE_FOUND);
            MSG_FILE_NOT_FOUND =         messages.getString(MSG_FILE_NOT_FOUND);
            MSG_NO_JAR_FILE_IN_DIR =     messages.getString(MSG_NO_JAR_FILE_IN_DIR);
            MSG_JVM_FOUND =              messages.getString(MSG_JVM_FOUND);
            MSG_JVM_NOT_FOUND =          messages.getString(MSG_JVM_NOT_FOUND);
            MSG_NXJ_FOUND =              messages.getString(MSG_NXJ_FOUND);
            MSG_NXJ_NOT_FOUND =          messages.getString(MSG_NXJ_NOT_FOUND);
            MSG_EV3_FOUND =              messages.getString(MSG_EV3_FOUND);
            MSG_EV3_NOT_FOUND =          messages.getString(MSG_EV3_NOT_FOUND);

            MSG_SHOW_DETAILS =           messages.getString(MSG_SHOW_DETAILS);
            MSG_INIT_DETAILS =           messages.getString(MSG_INIT_DETAILS);

            MSG_COMPILING_PACKAGE =      messages.getString(MSG_COMPILING_PACKAGE);
            MSG_COMPILING =              messages.getString(MSG_COMPILING);
            MSG_COMPILATION_RUNNING =    messages.getString(MSG_COMPILATION_RUNNING);
            MSG_COMPILATION_COMPLETED =  messages.getString(MSG_COMPILATION_COMPLETED);
            MSG_COMPILATION_FAILED =     messages.getString(MSG_COMPILATION_FAILED);

            MSG_JAR =                    messages.getString(MSG_JAR);
            MSG_JAR_COMPLETED =          messages.getString(MSG_JAR_COMPLETED);
            MSG_JAR_FAILED =             messages.getString(MSG_JAR_FAILED);

            MSG_SIMULATION =             messages.getString(MSG_SIMULATION);
            MSG_SIM_INIT =               messages.getString(MSG_SIM_INIT);
            MSG_SIM_READY =              messages.getString(MSG_SIM_READY);
            MSG_SIM_NOT_READY =          messages.getString(MSG_SIM_NOT_READY);
            MSG_SIM_START =              messages.getString(MSG_SIM_START);
            MSG_SIM_STARTED =            messages.getString(MSG_SIM_STARTED);
            MSG_SIM_RUNNING =            messages.getString(MSG_SIM_RUNNING);

            MSG_NXT =                    messages.getString(MSG_NXT);
            MSG_NXT_INIT =               messages.getString(MSG_NXT_INIT);
            MSG_NXT_READY =              messages.getString(MSG_NXT_READY);
            MSG_NXT_NOT_READY =          messages.getString(MSG_NXT_NOT_READY);
            MSG_NXT_BROWSER =            messages.getString(MSG_NXT_BROWSER);
            MSG_NXT_BROWSER_STARTED =    messages.getString(MSG_NXT_BROWSER_STARTED);
            MSG_NXT_BROWSER_RUNNING =    messages.getString(MSG_NXT_BROWSER_RUNNING);
            MSG_NXT_CONSOLE =            messages.getString(MSG_NXT_CONSOLE);
            MSG_NXT_CONSOLE_STARTED =    messages.getString(MSG_NXT_CONSOLE_STARTED);
            MSG_NXT_CONSOLE_RUNNING =    messages.getString(MSG_NXT_CONSOLE_RUNNING);
            MSG_NXT_FLASHTOOL =          messages.getString(MSG_NXT_FLASHTOOL);
            MSG_NXT_FLASHTOOL_STARTED =  messages.getString(MSG_NXT_FLASHTOOL_STARTED);
            MSG_NXT_FLASHTOOL_RUNNING =  messages.getString(MSG_NXT_FLASHTOOL_RUNNING);
            MSG_NXT_COMPILEPACKAGE =     messages.getString(MSG_NXT_COMPILEPACKAGE);
            MSG_NXT_COMPILE =            messages.getString(MSG_NXT_COMPILE);
            MSG_NXT_UPLOAD =             messages.getString(MSG_NXT_UPLOAD);
            MSG_NXT_UPLOAD_RUNNING =     messages.getString(MSG_NXT_UPLOAD_RUNNING);

            MSG_EV3 =                    messages.getString(MSG_EV3);
            MSG_EV3_INIT =               messages.getString(MSG_EV3_INIT);
            MSG_EV3_READY =              messages.getString(MSG_EV3_READY);
            MSG_EV3_NOT_READY =          messages.getString(MSG_EV3_NOT_READY);
            MSG_EV3_CONTROL =            messages.getString(MSG_EV3_CONTROL);
            MSG_EV3_CONTROL_STARTED =    messages.getString(MSG_EV3_CONTROL_STARTED);
            MSG_EV3_CONTROL_RUNNING =    messages.getString(MSG_EV3_CONTROL_RUNNING);
            MSG_EV3_ROBOT =              messages.getString(MSG_EV3_ROBOT);
            MSG_EV3_ROBOT_ADDRESS =      messages.getString(MSG_EV3_ROBOT_ADDRESS);
            MSG_EV3_SDCARD =             messages.getString(MSG_EV3_SDCARD);
            MSG_EV3_SDCARD_STARTED =     messages.getString(MSG_EV3_SDCARD_STARTED);
            MSG_EV3_SDCARD_RUNNING =     messages.getString(MSG_EV3_SDCARD_RUNNING);
            MSG_EV3_LIBS =               messages.getString(MSG_EV3_LIBS);
            MSG_EV3_LIBS_RUNNING =       messages.getString(MSG_EV3_LIBS_RUNNING);
            MSG_EV3_LIBS_COMPLETED =     messages.getString(MSG_EV3_LIBS_COMPLETED);
            MSG_EV3_LIBS_FAILED =        messages.getString(MSG_EV3_LIBS_FAILED);
            MSG_EV3_COMPILEPACKAGE =     messages.getString(MSG_EV3_COMPILEPACKAGE);
            MSG_EV3_COMPILE =            messages.getString(MSG_EV3_COMPILE);
            MSG_EV3_UPLOAD =             messages.getString(MSG_EV3_UPLOAD);
            MSG_EV3_UPLOAD_RUNNING =     messages.getString(MSG_EV3_UPLOAD_RUNNING);
            MSG_EV3_UPLOAD_COMPLETED =   messages.getString(MSG_EV3_UPLOAD_COMPLETED);
            MSG_EV3_UPLOAD_FAILED =      messages.getString(MSG_EV3_UPLOAD_FAILED);
        }
        catch (Exception e)
        {
            System.out.println ("Robot extension: Internationialization of messages failed. Details:\n" + e.getMessage());
        }
    }

    public void startup()
    {
        String language = mBlueJ.getBlueJPropertyString("bluej.language","english");
        if (language.equals("german"))
        {
            mLanguage = "de";
        }
        else
        {
            mLanguage = "en";
        }
        initMessages();

        mReadyNXT = true;
        mReadyEV3 = true;
        mReadySIM = true;
        //mBereit = true;
        mWindows = ('\\' == File.separatorChar);

        mPB = new ProcessBuilder(mCommandList);

        // Lade die benötigen Umgebungsvariablen
        Map<String,String> env = System.getenv();

        StringBuilder sb = new StringBuilder();


        sb.append("---------------------------------------------------------------\n");
        sb.append(MSG_SIM_INIT);
        sb.append("\n");
        sb.append("---------------------------------------------------------------\n");

        String bluejJavaHome = env.get("BLUEJ_JAVA_HOME");
        String bluejJava = "";
        if (null == bluejJavaHome)
        {
            bluejJavaHome = getSuitableJavaPath();
            if (null == bluejJavaHome)
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" BLUEJ_JAVA_HOME\n");
                sb.append(MSG_JVM_NOT_FOUND);
                sb.append("\n");
                mReadySIM = false;
            }
            else
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" BLUEJ_JAVA_HOME\n");
                sb.append(MSG_JVM_FOUND);
                sb.append(" ");
                sb.append(bluejJavaHome);
                sb.append("\n");
            }
        }
        else if (!(new File(bluejJavaHome)).isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_ENV_VAR_NOT_OK);
            sb.append(" BLUEJ_JAVA_HOME = ");
            sb.append(bluejJavaHome);
            sb.append("\n");
            mReadySIM = false;
        }
        else
        {
            sb.append(MSG_ENV_VAR_OK);
            sb.append(" BLUEJ_JAVA_HOME = ");
            sb.append(bluejJavaHome);
            sb.append("\n");
        }
        if (mReadySIM)
        {
            bluejJava = bluejJavaHome + File.separatorChar + "bin" + File.separatorChar + ((mWindows) ? "java.exe" : "java");
            if (!(new File(bluejJava)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(" ");
                sb.append(bluejJava);
                sb.append("\n");
                mReadySIM = false;
            }
        }

        String bluejUserlib = mBlueJ.getSystemLibDir().getAbsolutePath() + File.separatorChar + "userlib";
        File fileBluejUserLib = new File(bluejUserlib);
        if (!fileBluejUserLib.isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_DIR_NOT_FOUND);
            sb.append(" ");
            sb.append(bluejUserlib);
            sb.append("\n");
            mReadySIM = false;
        }
        else
        {
            sb.append(MSG_DIR_FOUND);
            sb.append(" ");
            sb.append(bluejUserlib);
            sb.append("\n");
            mClasspathUserlibBlueJ = buildClasspath(fileBluejUserLib);
            if (null == mClasspathUserlibBlueJ)
            {
                sb.append(MSG_ERROR);
                sb.append(": ");
                sb.append(MSG_NO_JAR_FILE_IN_DIR);
                sb.append("\n");
                mReadySIM = false;
            }
            else if (!mClasspathUserlibBlueJ.contains("jabotics-sim.jar"))
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(" jabotics-sim.jar\n");
                mReadySIM = false;
            }
            else
            {
                sb.append(MSG_FILE_FOUND);
                sb.append(" jabotics-sim.jar\n");
            }
        }

        sb.append("---------------------------------------------------------------\n");
        sb.append(MSG_NXT_INIT);
        sb.append("\n");
        sb.append("---------------------------------------------------------------\n");

        String nxjHome = env.get("NXJ_HOME");
        String nxjClasses = "";
        String nxjClasspathPC = "";
        String nxjLibPC;
        String nxjLibNXT;
        String nxjLibUser;
        File nxjFileLibPC;
        File nxjFileLibNXT;
        File nxjFileLibUser;
        if (null == nxjHome)
        {
            nxjHome = getSuitableLeJOSHome("leJOS_NXJ");
            if (null == nxjHome)
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" NXJ_HOME\n");
                sb.append(MSG_NXJ_NOT_FOUND);
                sb.append("\n");
                mReadyNXT = false;
            }
            else
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" NXJ_HOME\n");
                sb.append(MSG_NXJ_FOUND);
                sb.append(" ");
                sb.append(nxjHome);
                sb.append("\n");
            }
        }
        else if (!(new File(nxjHome)).isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_ENV_VAR_NOT_OK);
            sb.append(" NXJ_HOME = ");
            sb.append(nxjHome);
            sb.append("\n");
            mReadyNXT = false;
        }
        else
        {
            sb.append(MSG_ENV_VAR_OK);
            sb.append(" NXJ_HOME = ");
            sb.append(nxjHome);
            sb.append("\n");
        }

        if (mReadyNXT)
        {
            nxjLibNXT = nxjHome + File.separatorChar + "lib" + File.separatorChar + "nxt";
            nxjFileLibNXT = new File(nxjLibNXT);
            if (nxjFileLibNXT.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(" ");
                sb.append(nxjLibNXT);
                sb.append("\n");
                nxjClasses = nxjLibNXT + File.separatorChar + "classes.jar";
                if (!(new File(nxjClasses)).isFile())
                {
                    sb.append(MSG_ERROR);
                    sb.append(" - ");
                    sb.append(MSG_FILE_NOT_FOUND);
                    sb.append(" ");
                    sb.append(nxjClasses);
                    sb.append("\n");
                    mReadyNXT = false;
                }
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(" ");
                sb.append(nxjLibNXT);
                sb.append("\n");
                mReadyNXT = false;
            }
    
            nxjLibPC = nxjHome + File.separatorChar + "lib" + File.separatorChar + "pc";
            nxjFileLibPC = new File(nxjLibPC);
            if (nxjFileLibPC.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(" ");
                sb.append(nxjLibPC);
                sb.append("\n");
                nxjClasspathPC = buildClasspath(nxjFileLibPC);
                if (null == nxjClasspathPC)
                {
                    sb.append(MSG_ERROR);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append("\n");
                    mReadyNXT = false;
                }
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(" ");
                sb.append(nxjLibPC);
                sb.append("\n");
                mReadyNXT = false;
            }
    
            nxjLibUser = nxjHome + File.separatorChar + "lib" + File.separatorChar + "userlib";
            nxjFileLibUser = new File(nxjLibUser);
            if (nxjFileLibUser.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(" ");
                sb.append(nxjLibUser);
                sb.append("\n");
                mClasspathUserlibNXJ = buildClasspath(nxjFileLibUser);
                if (null == mClasspathUserlibNXJ)
                {
                    sb.append(MSG_WARNING);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append("\n");
                }
            }
            else
            {
                sb.append(MSG_WARNING);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(" ");
                sb.append(nxjLibUser);
                sb.append("\n");
            }
        }

        String nxjJavaHome = env.get("LEJOS_NXJ_JAVA_HOME");
        String nxjJava = "";
        String nxjJavac = "";
        if (null == nxjJavaHome)
        {
            nxjJavaHome = bluejJavaHome;

            if (null == nxjJavaHome)
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" LEJOS_NXJ_JAVA_HOME\n");
                sb.append(MSG_JVM_NOT_FOUND);
                sb.append("\n");
                mReadyNXT = false;
            }
            else
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" LEJOS_NXJ_JAVA_HOME\n");
                sb.append(MSG_JVM_FOUND);
                sb.append(" ");
                sb.append(nxjJavaHome);
                sb.append("\n");
            }
        }
        else if (!(new File(nxjJavaHome)).isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_ENV_VAR_NOT_OK);
            sb.append(" LEJOS_NXJ_JAVA_HOME = ");
            sb.append(nxjJavaHome);
            sb.append("\n");
            mReadyNXT = false;
        }
        else
        {
            sb.append(MSG_ENV_VAR_OK);
            sb.append(" LEJOS_NXJ_JAVA_HOME = ");
            sb.append(nxjJavaHome);
            sb.append("\n");
        }
        if (mReadyNXT)
        {
            nxjJava = nxjJavaHome + File.separatorChar + "bin" + File.separatorChar + ((mWindows) ? "java.exe" : "java");
            if (!(new File(nxjJava)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(" ");
                sb.append(nxjJava);
                sb.append("\n");
                mReadyNXT = false;
            }
    
            nxjJavac = nxjJavaHome + File.separatorChar + "bin" + File.separatorChar + ((mWindows) ? "javac.exe" : "javac");
            if (!(new File(nxjJavac)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(" ");
                sb.append(nxjJavac);
                sb.append("\n");
                mReadyNXT = false;
            }
        }


        sb.append("---------------------------------------------------------------\n");
        sb.append(MSG_EV3_INIT);
        sb.append("\n");
        sb.append("---------------------------------------------------------------\n");

        String ev3Home = env.get("EV3_HOME");
        String ev3ClasspathPC = "";
        String ev3ClasspathUserlib = "";
        String ev3LibEV3;
        String ev3LibPC;
        String ev3LibUser = "";
        File ev3FileLibPC;
        File ev3FileLibEV3;
        File ev3FileLibUser;
        if (null == ev3Home)
        {
            ev3Home = getSuitableLeJOSHome("leJOS_EV3");
            if (null == ev3Home)
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" EV3_HOME\n");
                sb.append(MSG_EV3_NOT_FOUND);
                sb.append("\n");
                mReadyEV3 = false;
            }
            else
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" EV3_HOME\n");
                sb.append(MSG_EV3_FOUND);
                sb.append(" ");
                sb.append(ev3Home);
                sb.append("\n");
            }
        }
        else if (!(new File(ev3Home)).isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_ENV_VAR_NOT_OK);
            sb.append(" EV3_HOME = ");
            sb.append(ev3Home);
            sb.append("\n");
            mReadyEV3 = false;
        }
        else
        {
            sb.append(MSG_ENV_VAR_OK);
            sb.append(" EV3_HOME = ");
            sb.append(ev3Home);
            sb.append("\n");
        }

        if (mReadyEV3)
        {
            ev3LibEV3 = ev3Home + File.separatorChar + "lib" + File.separatorChar + "ev3";
            ev3FileLibEV3 = new File(ev3LibEV3);
            if (ev3FileLibEV3.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(" ");
                sb.append(ev3LibEV3);
                sb.append("\n");
                mClasspathEV3 = buildClasspath(ev3FileLibEV3);
                if (null == mClasspathEV3)
                {
                    sb.append(MSG_ERROR);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append("\n");
                    mReadyEV3 = false;
                }
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(" ");
                sb.append(ev3LibEV3);
                sb.append("\n");
                mReadyEV3 = false;
            }
        }

        if (mReadyEV3)
        {
            ev3LibPC = ev3Home + File.separatorChar + "lib" + File.separatorChar + "pc";
            ev3FileLibPC = new File(ev3LibPC);
            if (ev3FileLibPC.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(" ");
                sb.append(ev3LibPC);
                sb.append("\n");
                ev3ClasspathPC = buildClasspath(ev3FileLibPC);
                if (null == ev3ClasspathPC)
                {
                    sb.append(MSG_ERROR);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append("\n");
                    mReadyEV3 = false;
                }
                else
                {
                    // some PC tools require binaries from ev3classes.jar
                    ev3ClasspathPC = ev3ClasspathPC + File.pathSeparator + mClasspathEV3;
                }
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(" ");
                sb.append(ev3LibPC);
                sb.append("\n");
                mReadyEV3 = false;
            }
        }

        if (mReadyEV3)
        {
            ev3LibUser = ev3Home + File.separatorChar + "lib" + File.separatorChar + "userlib";
            ev3FileLibUser = new File(ev3LibUser);
            if (ev3FileLibUser.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(" ");
                sb.append(ev3LibUser);
                sb.append("\n");
                ev3ClasspathUserlib = buildClasspath(ev3FileLibUser,mFilesUserlibEV3);
                if (null == ev3ClasspathUserlib)
                {
                    sb.append(MSG_WARNING);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append("\n");
                }
                else
                {
                    mClasspathEV3 = mClasspathEV3 + File.pathSeparator + ev3ClasspathUserlib;
                }
            }
            else
            {
                sb.append(MSG_WARNING);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(" ");
                sb.append(ev3LibUser);
                sb.append("\n");
            }
        }

        String ev3JavaHome = env.get("LEJOS_EV3_JAVA_HOME");
        String ev3Java = "";
        String ev3Javac = "";
        String ev3Jar = "";
        String ev3RT = "";  // Dummy runtime to avoid compiler warning for missing bootclasspath
        if (null == ev3JavaHome)
        {
            ev3JavaHome = bluejJavaHome;

            if (null == ev3JavaHome)
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" LEJOS_EV3_JAVA_HOME\n");
                sb.append(MSG_JVM_NOT_FOUND);
                sb.append("\n");
                mReadyEV3 = false;
            }
            else
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" LEJOS_EV3_JAVA_HOME\n");
                sb.append(MSG_JVM_FOUND);
                sb.append(" ");
                sb.append(ev3JavaHome);
                sb.append("\n");
            }
        }
        else if (!(new File(ev3JavaHome)).isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_ENV_VAR_NOT_OK);
            sb.append(" LEJOS_EV3_JAVA_HOME = ");
            sb.append(ev3JavaHome);
            sb.append("\n");
            mReadyEV3 = false;
        }
        else
        {
            sb.append(MSG_ENV_VAR_OK);
            sb.append(" LEJOS_EV3_JAVA_HOME = ");
            sb.append(ev3JavaHome);
            sb.append("\n");
        }
        if (mReadyEV3)
        {
            ev3Java = ev3JavaHome + File.separatorChar + "bin" + File.separatorChar + ((mWindows) ? "java.exe" : "java");
            if (!(new File(ev3Java)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(" ");
                sb.append(ev3Java);
                sb.append("\n");
                mReadyEV3 = false;
            }

            ev3Javac = ev3JavaHome + File.separatorChar + "bin" + File.separatorChar + ((mWindows) ? "javac.exe" : "javac");
            if (!(new File(ev3Javac)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(" ");
                sb.append(ev3Javac);
                sb.append("\n");
                mReadyEV3 = false;
            }

            ev3Jar = ev3JavaHome + File.separatorChar + "bin" + File.separatorChar + ((mWindows) ? "jar.exe" : "jar");
            if (!(new File(ev3Jar)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(" ");
                sb.append(ev3Jar);
                sb.append("\n");
                mReadyEV3 = false;
            }

            ev3RT = ev3JavaHome + File.separatorChar + "jre" + File.separatorChar + "lib" + File.separatorChar + "rt.jar";

        }

        mInitDetails = sb.toString();
        sb.setLength(0);

        sb.append(MSG_VERSION);
        sb.append(": ");
        sb.append(VERSION_NUMBER);
        sb.append("\n");
        sb.append(MSG_AUTHOR);
        sb.append(": Frank Gollnik\n");
        sb.append("www.jabotics.org (©");
        sb.append(COPYRIGHT);
        sb.append(")\n");
        
        if (mReadySIM)
        {
            sb.append(MSG_SIM_READY);

            mSimulateCommandList.add(bluejJava);
            mSimulateCommandList.add("-classpath");
        }
        else
        {
            sb.append(MSG_SIM_NOT_READY);
        }

        if (mReadyNXT)
        {
            sb.append("\n");
            sb.append(MSG_NXT_READY);

            mBrowserCommandListNXT.add(nxjJava);
            mBrowserCommandListNXT.add("-Dnxj.home=" + nxjHome);
            mBrowserCommandListNXT.add("-DCOMMAND_NAME=nxjbrowse");
            mBrowserCommandListNXT.add("-classpath");
            mBrowserCommandListNXT.add(nxjClasspathPC);
            mBrowserCommandListNXT.add("lejos.pc.tools.NXJBrowser");
            
            mConsoleCommandListNXT.add(nxjJava);
            mConsoleCommandListNXT.add("-Dnxj.home=" + nxjHome);
            mConsoleCommandListNXT.add("-DCOMMAND_NAME=nxjconsoleviewer");
            mConsoleCommandListNXT.add("-classpath");
            mConsoleCommandListNXT.add(nxjClasspathPC);
            mConsoleCommandListNXT.add("lejos.pc.tools.NXJConsoleViewer");
    
            mFlashCommandListNXT.add(nxjJava);
            mFlashCommandListNXT.add("-Dnxj.home=" + nxjHome);
            mFlashCommandListNXT.add("-DCOMMAND_NAME=nxjflashg");
            mFlashCommandListNXT.add("-classpath");
            mFlashCommandListNXT.add(nxjClasspathPC);
            mFlashCommandListNXT.add("lejos.pc.tools.NXJFlashG");
    
            mCompileCommandListNXT.add(nxjJavac);
            mCompileCommandListNXT.add("-source");
            mCompileCommandListNXT.add("1.6");
            mCompileCommandListNXT.add("-target");
            mCompileCommandListNXT.add("1.6");
            mCompileCommandListNXT.add("-bootclasspath");
            mCompileCommandListNXT.add(nxjClasses);
            if (null != mClasspathUserlibNXJ)
            {
                mCompileCommandListNXT.add("-classpath");
                mCompileCommandListNXT.add(mClasspathUserlibNXJ);
            }
            //mCompileCommandListNXT.add("-implicit:class");
            mCompileCommandListNXT.add("-sourcepath");
    
            mLinkAndUploadCommandListNXT.add(nxjJava);
            mLinkAndUploadCommandListNXT.add("-Dnxj.home=" + nxjHome);
            mLinkAndUploadCommandListNXT.add("-DCOMMAND_NAME=nxjlinkandupload");
            mLinkAndUploadCommandListNXT.add("-classpath");
            mLinkAndUploadCommandListNXT.add(nxjClasspathPC);
            mLinkAndUploadCommandListNXT.add("lejos.pc.tools.NXJLinkAndUpload");
            //mLinkAndUploadCommandListNXT.add("--usb");
            mLinkAndUploadCommandListNXT.add("--verbose");
            mLinkAndUploadCommandListNXT.add("--writeorder");
            mLinkAndUploadCommandListNXT.add("LE");
            mLinkAndUploadCommandListNXT.add("--bootclasspath");
            mLinkAndUploadCommandListNXT.add(nxjClasses);
            mLinkAndUploadCommandListNXT.add("--classpath");
        }
        else
        {
            sb.append("\n");
            sb.append(MSG_NXT_NOT_READY);
        }

        if (mReadyEV3)
        {
            sb.append("\n");
            sb.append(MSG_EV3_READY);

            mControlCommandListEV3.add(ev3Java);
            mControlCommandListEV3.add("-Dev3.home=" + ev3Home);
            mControlCommandListEV3.add("-DCOMMAND_NAME=ev3control");
            mControlCommandListEV3.add("-classpath");
            mControlCommandListEV3.add(ev3ClasspathPC);
            mControlCommandListEV3.add("lejos.ev3.tools.EV3Control");

            mSDCardCommandListEV3.add(ev3Java);
            mSDCardCommandListEV3.add("-Dev3.home=" + ev3Home);
            mSDCardCommandListEV3.add("-DCOMMAND_NAME=ev3sdcard");
            mSDCardCommandListEV3.add("-classpath");
            mSDCardCommandListEV3.add(buildClasspath(new File(mBlueJ.getSystemLibDir().getAbsolutePath() + File.separatorChar + "extensions")));
            mSDCardCommandListEV3.add("org.jabotics.robot.bluej.EV3SDCard");

            mLibUploadCommandListEV3.add(ev3Java);
            mLibUploadCommandListEV3.add("-Dev3.home=" + ev3Home);
            mLibUploadCommandListEV3.add("-DCOMMAND_NAME=ev3scpupload");
            mLibUploadCommandListEV3.add("-classpath");
            mLibUploadCommandListEV3.add(ev3ClasspathPC);
            mLibUploadCommandListEV3.add("lejos.ev3.tools.EV3ScpUpload");
            mLibUploadCommandListEV3.add("-n");

            mCompileCommandListEV3.add(ev3Javac);
            mCompileCommandListEV3.add("-source");
            mCompileCommandListEV3.add("1.7");
            mCompileCommandListEV3.add("-target");
            mCompileCommandListEV3.add("1.7");
            mCompileCommandListEV3.add("-bootclasspath");
            mCompileCommandListEV3.add(ev3RT);  // Dummy runtime to avoid compiler warning for missing bootclasspath
            mCompileCommandListEV3.add("-classpath");
            mCompileCommandListEV3.add(mClasspathEV3);
            //mCompileCommandListEV3.add("-implicit:class");
            mCompileCommandListEV3.add("-sourcepath");

            mJarCommandListEV3.add(ev3Jar);
            //mJarCommandListEV3.add("-c0mfe"); // uncompressed: does not seem to speed up start procedure
            mJarCommandListEV3.add("-cmfe");
            mJarCommandListEV3.add(ev3LibUser + File.separatorChar + "manifest-ev3");
    
            mUploadCommandListEV3.add(ev3Java);
            mUploadCommandListEV3.add("-Dev3.home=" + ev3Home);
            mUploadCommandListEV3.add("-DCOMMAND_NAME=ev3scpupload");
            mUploadCommandListEV3.add("-classpath");
            mUploadCommandListEV3.add(ev3ClasspathPC);
            mUploadCommandListEV3.add("lejos.ev3.tools.EV3ScpUpload");
            mUploadCommandListEV3.add("-n");
        }
        else
        {
            sb.append("\n");
            sb.append(MSG_EV3_NOT_READY);
        }

        mInfo = sb.toString();
        sb.setLength(0);
    }

    static private class DirectoryFilterByPrefix implements FileFilter
    {
        String mPrefix;
        
        DirectoryFilterByPrefix(String prefix)
        {
            mPrefix = prefix;
        }
        
        public boolean accept(File file)
        {
            return file.isDirectory() && file.getName().startsWith(mPrefix);
        }
    }

    private String getSuitableJavaPath()
    {
        if (mWindows)
        {
            // ToDo: Needs to be tested for Win32 and Win64! Probably needs to be corrected / improved for Win64.
            File jvmDir = new File("C:\\Program Files (x86)\\BlueJ\\jdk");
            if (!jvmDir.exists() || !jvmDir.isDirectory())
            {
                jvmDir = new File("C:\\Program Files\\BlueJ\\jdk");
                if (!jvmDir.exists() || !jvmDir.isDirectory()) jvmDir = null;
            }
            if (null != jvmDir)
            {
                File toolsJar = new File(jvmDir.getAbsolutePath() + "\\lib\\tools.jar");
                if (toolsJar.exists() && toolsJar.isFile()) return jvmDir.getAbsolutePath();
            }

            jvmDir = new File("C:\\Program Files (x86)\\Java");
            if (!jvmDir.exists() || !jvmDir.isDirectory())
            {
                jvmDir = new File("C:\\Program Files\\Java");
                if (!jvmDir.exists() || !jvmDir.isDirectory()) return null;
            }

            for (int i = 0;  i < VM_NAMES_WIN.length;  ++i)
            {
                File[] files = jvmDir.listFiles(new DirectoryFilterByPrefix(VM_NAMES_WIN[i]));
                for (File file : files)
                {
                    File toolsJar = new File(file.getAbsolutePath() + "\\lib\\tools.jar");
                    if (toolsJar.exists() && toolsJar.isFile()) return file.getAbsolutePath();
                }
            }
            return null;
        }
        else
        {
            File jvmDir = new File("/usr/lib/jvm");
            if (!jvmDir.exists() || !jvmDir.isDirectory()) return null;

            for (int i = 0;  i < VM_NAMES_LINUX.length;  ++i)
            {
                File[] files = jvmDir.listFiles(new DirectoryFilterByPrefix(VM_NAMES_LINUX[i]));
                for (File file : files)
                {
                    File toolsJar = new File(file.getAbsolutePath() + "/lib/tools.jar");
                    if (toolsJar.exists() && toolsJar.isFile()) return file.getAbsolutePath();
                }
            }

            return null;
        }
    }

    private String getSuitableLeJOSHome(String leJOS_Version)
    {
        // ToDo: Needs to be tested for Win32 and Win64!

        // Linux?
        if (!mWindows) return getSuitableLeJOSHome("/opt",leJOS_Version);

        // Win64?
        String home = getSuitableLeJOSHome("C:\\Program Files (x86)",leJOS_Version);
        if (null != home) return home;

        // Win32?
        return getSuitableLeJOSHome("C:\\Program Files",leJOS_Version);
    }

    static private String getSuitableLeJOSHome(String dirName, String leJOS_Version)
    {
        File dir = new File(dirName);
        if (!dir.exists() || !dir.isDirectory()) return null;

        String home = dirName + File.separatorChar + leJOS_Version;
        File homeDir = new File(home);
        if (homeDir.exists() && homeDir.isDirectory()) return home;

        File[] files = dir.listFiles(new DirectoryFilterByPrefix(leJOS_Version));
        for (File file : files)
        {
            return file.getAbsolutePath();
        }

        return null;
    }

    static private String buildClasspath(File dir)
    {
        ArrayList<File> fileList = new ArrayList<File>();
        return buildClasspath(dir, fileList);
    }

    static private String buildClasspath(File dir, ArrayList<File> fileList)
    {
        getJavaBinaries(fileList, dir, true);
        if (fileList.isEmpty()) return null;
        StringBuilder sb = new StringBuilder();
        for(File item : fileList)
        {
            sb.append(File.pathSeparator);  // ";" für Windows und ":" für Unix
            sb.append(item.getAbsolutePath());
        }
        sb.deleteCharAt(0);
        return sb.toString();
    }

    static private void getJavaBinaries(ArrayList<File> fileList, File dir, boolean recursive)
    {
        File[] files = dir.listFiles();
        for (File item : files)
        {
            if (item.isFile())
            {
                String lowerCaseName = item.getName().toLowerCase();
                if (lowerCaseName.endsWith(".jar") || lowerCaseName.endsWith(".class"))
                {
                    fileList.add(item);
                }
            }
            else
            {
                if (recursive && item.isDirectory()) getJavaBinaries(fileList, item, recursive);
            }
        }
    }


     /*
     * This method must decide if this Extension is compatible with the 
     * current release of the BlueJ Extensions API
     */
    public boolean isCompatible ()
    { 
        return true; 
    }

    /*
     * Returns the version number of this extension
     */
    public String  getVersion ()
    { 
        return (VERSION_NUMBER);  
    }

    /*
     * Returns the user-visible name of this extension
     */
    public String  getName ()
    { 
        return (MSG_ROBOT_EXT);  
    }

    public void terminate()
    {
        //System.out.println ("Robot extension terminates");
    }
    
    public String getDescription ()
    {
        return (MSG_ROBOT_EXT_INFO);
    }

    /*
     * Returns a URL where you can find info on this extension.
     * The real problem is making sure that the link will still be alive in three years...
     */
    public URL getURL ()
    {
        try
        {
            return new URL("http://www.jabotics.org");
        }
        catch ( Exception eee )
        {
            // The link is either dead or otherwise unreachable
            System.out.println (MSG_URL_ERROR + ":\n" + eee.getMessage());
            return null;
        }
    }
    
    /*
     * This nested class implements the preference panel behaviour for a BlueJ extension
     */
    class XPreferences implements PreferenceGenerator, ActionListener
    {
        private JPanel mPanel;
        private JButton mButtonDetails;
        private JDialog mDialog;
        private BlueJ mBlueJ;
    
        // Construct the panel, and initialise it from any stored values
        public XPreferences(BlueJ bluej) 
        {
            mBlueJ = bluej;
            mPanel = new JPanel(new BorderLayout());
            JTextArea textArea = new JTextArea(mInfo);
            textArea.setEditable(false);
            textArea.setFont(mPanel.getFont());
            textArea.setBackground(mPanel.getBackground());
            mPanel.add(textArea,BorderLayout.CENTER);
            mButtonDetails = new JButton(MSG_SHOW_DETAILS);
            mButtonDetails.addActionListener(this);
            mPanel.add(mButtonDetails,BorderLayout.SOUTH);
        }
    
        public JPanel getPanel ()  { return mPanel; }

        public void loadValues () {}

        public void saveValues () {}

        public void actionPerformed(ActionEvent e)
        {
            if (null == mDialog)
            {
                mDialog = new JDialog(mBlueJ.getCurrentFrame(),MSG_INIT_DETAILS,true);
                JPanel content = new JPanel(new BorderLayout());
                JTextArea textArea = new JTextArea(mInitDetails);
                textArea.setEditable(false);
                JScrollPane scrollPane = new JScrollPane(textArea);
                content.add(scrollPane,BorderLayout.CENTER);
                scrollPane.setPreferredSize(new Dimension(500,300));
                content.add(scrollPane);
                JButton button = new JButton("OK");
                button.addActionListener(this);
                content.add(button,BorderLayout.SOUTH);
                mDialog.add(content);
                mDialog.pack();
                mDialog.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
            }

            mDialog.setVisible(e.getSource().equals(mButtonDetails));
        }
    }

    class XReentrantLock extends ReentrantLock
    {
        // Eigene abgeleitete Klasse sichert Zugriff auf protected-Methode getOwner
        XReentrantLock()
        {
            super();
        }
        
        String getOwnerName()
        {
            Thread thread = this.getOwner();
            if (null == thread) return null;
            try
            {
                return thread.getName(); // Vermutlich kann hier gar nichts schiefgehen.
            }
            catch (Exception e) {}
            return null;
        }
    }
    
    /* Diese Klasse generiert die Menüeinträge für die Roboter-Erweiterung
     * Remember:
     * - getToolsMenuItem, getClassMenuItem and getObjectMenuItem may be called by BlueJ at any time.
     * - They must generate a new JMenuItem each time they are called.
     * - No reference to the JMenuItem should be stored in the extension.
     * - You must be quick in generating your menu.
     */
    class XMenuGenerator extends MenuGenerator
    {
        BClass mClass;
        BPackage mPackage;
        BProject mProject;
        XReentrantLock mConnectionLock;
        XReentrantLock mCompileLock;
        
        public XMenuGenerator()
        {
            super();
            mConnectionLock = new XReentrantLock();
            mCompileLock = new XReentrantLock();
        }
    
        public JMenuItem getToolsMenuItem(BPackage aPackage)
        {
            /*JMenu menu = new JMenu(MSG_ROBOT);
            menu.add(new JMenuItem(new NXTBrowserAction()));
            menu.add(new JMenuItem(new NXTConsoleAction()));
            menu.add(new JMenuItem(new NXTFlashAction()));
            menu.add(new JMenuItem(new NXTCompilePackageAction()));
            menu.addSeparator();
            menu.add(new JMenuItem(new EV3SelectRobotAction()));
            menu.add(new JMenuItem(new EV3ControlAction()));
            menu.add(new JMenuItem(new EV3SDCardAction()));
            menu.add(new JMenuItem(new EV3UserlibsUploadAction()));
            menu.add(new JMenuItem(new EV3CompilePackageAction()));
            return menu;*/

            JMenuItem item;
            JMenu menu = new JMenu(MSG_ROBOT);
            menu.setEnabled(mReadyNXT||mReadyEV3);

            item = new JMenuItem(new NXTBrowserAction());
            item.setEnabled(mReadyNXT);
            menu.add(item);

            item = new JMenuItem(new NXTConsoleAction());
            item.setEnabled(mReadyNXT);
            menu.add(item);

            item = new JMenuItem(new NXTFlashAction());
            item.setEnabled(mReadyNXT);
            menu.add(item);

            item = new JMenuItem(new NXTCompilePackageAction());
            item.setEnabled(mReadyNXT);
            menu.add(item);

            menu.addSeparator();

            item = new JMenuItem(new EV3SelectRobotAction());
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3ControlAction());
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3SDCardAction());
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3UserlibsUploadAction());
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3CompilePackageAction());
            item.setEnabled(mReadyEV3);
            menu.add(item);

            return menu;
        }
    
        public JMenuItem getClassMenuItem(BClass aClass)
        {
            /*JMenu menu = new JMenu(MSG_ROBOT);
            menu.add(new JMenuItem(new SimulateAction()));
            menu.addSeparator();
            menu.add(new JMenuItem(new NXTCompileAction()));
            menu.add(new JMenuItem(new NXTUploadAction()));
            menu.addSeparator();
            menu.add(new JMenuItem(new EV3CompileAction()));
            menu.add(new JMenuItem(new EV3UploadAction()));
            return menu;*/

            JMenuItem item;
            JMenu menu = new JMenu(MSG_ROBOT);
            menu.setEnabled(mReadySIM||mReadyNXT||mReadyEV3);

            item = new JMenuItem(new SimulateAction());
            item.setEnabled(mReadySIM);
            menu.add(item);

            menu.addSeparator();

            item = new JMenuItem(new NXTCompileAction());
            item.setEnabled(mReadyNXT);
            menu.add(item);

            item = new JMenuItem(new NXTUploadAction());
            item.setEnabled(mReadyNXT);
            menu.add(item);

            menu.addSeparator();

            item = new JMenuItem(new EV3CompileAction());
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3UploadAction());
            item.setEnabled(mReadyEV3 && !mFilesUserlibEV3.isEmpty());
            menu.add(item);

            return menu;
        }


        public void notifyPostToolsMenu(BPackage bp, JMenuItem jmi)
        {
            try
            {
                mClass = null;
                mPackage = bp;
                mProject = mPackage.getProject();
                if (jmi instanceof JMenu)
                {
                    // robot (root) menu item
                    jmi.setEnabled(mReadyNXT||mReadyEV3);
                }
                else
                {
                    Action action = jmi.getAction();
                    if ((action instanceof NXTBrowserAction) || (action instanceof NXTConsoleAction) || (action instanceof NXTFlashAction) || (action instanceof NXTCompilePackageAction))
                    {
                        jmi.setEnabled(mReadyNXT);
                    }
                    else
                    {
                        if (action instanceof EV3UserlibsUploadAction)
                        {
                            jmi.setEnabled(mReadyEV3 && (!mFilesUserlibEV3.isEmpty()));
                        }
                        else
                        {
                            jmi.setEnabled(mReadyEV3);
                        }
                    }
                }
            }
            catch (ProjectNotOpenException e)
            {
                jmi.setEnabled(false);
            }
        }

        public void notifyPostClassMenu(BClass bc, JMenuItem jmi)
        {
            try
            {
                mClass = bc;
                mPackage = bc.getPackage();
                mProject = mPackage.getProject();
                if (jmi instanceof JMenu)
                {
                    jmi.setEnabled(true);
                }
                else
                {
                    Action action = jmi.getAction();
                    if (action instanceof SimulateAction)
                    {
                        jmi.setEnabled(mReadySIM);
                    }
                    else if ((action instanceof NXTCompileAction) || (action instanceof NXTUploadAction))
                    {
                        jmi.setEnabled(mReadyNXT);
                    }
                    else
                    {
                        jmi.setEnabled(mReadyEV3);
                    }
                }
            }
            catch (ProjectNotOpenException e)
            {
                jmi.setEnabled(false);
            }
            catch (PackageNotFoundException e)
            {
                jmi.setEnabled(false);
            }
        }

        abstract class XAction extends AbstractAction implements Runnable
        {
            ArrayList<XReentrantLock> mLocks;
            String mType;

            XAction(String type, String name)
            {
                super();
                mLocks = new ArrayList<XReentrantLock>();
                mType = type;
                putValue(AbstractAction.NAME, name);
            }

            void addLock(XReentrantLock lock)
            {
                mLocks.add(lock);
            }

            public void actionPerformed(ActionEvent e)
            {
                (new Thread(this, mType)).start();
            }

            public void run()
            {
                aquireLock(0);
            }
            
            void aquireLock(int n)
            {
                if (mLocks.size() <= n)
                {
                    // no (further) lock to aquire
                    execute();
                    return;
                }

                XReentrantLock lock = mLocks.get(n);
                String threadname;
                while (null == (threadname = lock.getOwnerName()))
                {
                    if (!lock.tryLock()) continue;

                    try
                    {
                        // aquire next lock
                        aquireLock(n+1);
                    }
                    finally
                    {
                        lock.unlock();
                    }
                    
                    return;
                }

                try
                {
                    if (threadname.equals(mType))
                    {
                        if (threadname.equals(NXT_BROWSER_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_NXT_BROWSER_STARTED);
                            return;
                        }
                        if (threadname.equals(NXT_CONSOLE_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_NXT_CONSOLE_STARTED);
                            return;
                        }
                        if (threadname.equals(NXT_FLASH_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_NXT_FLASHTOOL_STARTED);
                            return;
                        }
                        if (threadname.equals(NXT_COMPILE_ACTION) || threadname.equals(NXT_COMPILEPACKAGE_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_COMPILATION_RUNNING);
                            return;
                        }
                        if (threadname.equals(NXT_UPLOAD_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_NXT_UPLOAD_RUNNING);
                            return;
                        }
                        if (threadname.equals(EV3_CONTROL_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_EV3_CONTROL_STARTED);
                            return;
                        }
                        if (threadname.equals(EV3_SDCARD_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_EV3_SDCARD_STARTED);
                            return;
                        }
                        if (threadname.equals(EV3_LIBS_UPLOAD_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_EV3_LIBS_RUNNING);
                            return;
                        }
                        if (threadname.equals(EV3_COMPILE_ACTION) || threadname.equals(EV3_COMPILEPACKAGE_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_COMPILATION_RUNNING);
                            return;
                        }
                        if (threadname.equals(EV3_UPLOAD_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_EV3_UPLOAD_RUNNING);
                            return;
                        }
                        if (threadname.equals(SIMULATE_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_SIM_STARTED);
                            return;
                        }
                    }
                    else
                    {
                        if (threadname.equals(NXT_BROWSER_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_NXT_BROWSER_RUNNING);
                            return;
                        }
                        if (threadname.equals(NXT_CONSOLE_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_NXT_CONSOLE_RUNNING);
                            return;
                        }
                        if (threadname.equals(NXT_FLASH_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_NXT_FLASHTOOL_RUNNING);
                            return;
                        }
                        if (threadname.equals(NXT_COMPILE_ACTION) || threadname.equals(NXT_COMPILEPACKAGE_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_COMPILATION_RUNNING);
                            return;
                        }
                        if (threadname.equals(NXT_UPLOAD_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_NXT_UPLOAD_RUNNING);
                            return;
                        }
                        if (threadname.equals(EV3_CONTROL_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_EV3_CONTROL_RUNNING);
                            return;
                        }
                        if (threadname.equals(EV3_SDCARD_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_EV3_SDCARD_RUNNING);
                            return;
                        }
                        if (threadname.equals(EV3_LIBS_UPLOAD_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_EV3_LIBS_RUNNING);
                            return;
                        }
                        if (threadname.equals(EV3_COMPILE_ACTION) || threadname.equals(EV3_COMPILEPACKAGE_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_COMPILATION_RUNNING);
                            return;
                        }
                        if (threadname.equals(EV3_UPLOAD_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_EV3_UPLOAD_RUNNING);
                            return;
                        }
                        if (threadname.equals(SIMULATE_ACTION))
                        {
                            JOptionPane.showMessageDialog(mPackage.getFrame(), MSG_SIM_RUNNING);
                            return;
                        }
                    }
                }
                catch (ProjectNotOpenException pnoEx) {}
                catch (PackageNotFoundException pnfEx) {}
            }

            abstract public void execute();
        }

        class NXTBrowserAction extends XAction
        {
            public NXTBrowserAction()
            {
                super(NXT_BROWSER_ACTION,MSG_NXT_BROWSER);
                addLock(mConnectionLock);
            }
            
            public void execute()
            {
                try
                {
                    mCommandList.clear();
                    mCommandList.addAll(mBrowserCommandListNXT);
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_BROWSER);
                    viewer.showOnError(true);
                    viewer.automaticTermination(true);
                    viewer.startLogging(p);
                    p.waitFor();
                }
                catch (IOException ioEx) {}
                catch (InterruptedException irEx) {}
            }
        }

        class NXTConsoleAction extends XAction
        {
            public NXTConsoleAction()
            {
                super(NXT_CONSOLE_ACTION,MSG_NXT_CONSOLE);
                addLock(mConnectionLock);
            }

            public void execute()
            {
                try
                {
                    mCommandList.clear();
                    mCommandList.addAll(mConsoleCommandListNXT);
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_CONSOLE);
                    viewer.showOnError(true);
                    viewer.automaticTermination(true);
                    viewer.startLogging(p);
                    p.waitFor();
                }
                catch (IOException ioEx) {}
                catch (InterruptedException irEx) {}
            }
        }

        class NXTFlashAction extends XAction
        {
            public NXTFlashAction()
            {
                super(NXT_FLASH_ACTION,MSG_NXT_FLASHTOOL);
                addLock(mConnectionLock);
            }

            public void execute()
            {
                try
                {
                    mCommandList.clear();
                    mCommandList.addAll(mFlashCommandListNXT);
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_FLASHTOOL);
                    viewer.showOnError(true);
                    viewer.automaticTermination(true);
                    viewer.startLogging(p);
                    p.waitFor();
                }
                catch (IOException ioEx) {}
                catch (InterruptedException irEx) {}
            }
        }

        class NXTCompilePackageAction extends XAction
        {
            public NXTCompilePackageAction()
            {
                super(NXT_COMPILEPACKAGE_ACTION,MSG_NXT_COMPILEPACKAGE);
                addLock(mCompileLock);
            }
            
            public void execute()
            {
                try
                {
                    mCommandList.clear();
                    mCommandList.addAll(mCompileCommandListNXT);
                    mCommandList.add(mProject.getDir().getAbsolutePath());  // SOURCEPATH
                    File[] files = mPackage.getDir().listFiles();  // look for java files in package directory
                    for (File item : files)
                    {
                        if (item.isFile())
                        {
                            String lowerCaseName = item.getName().toLowerCase();
                            if (lowerCaseName.endsWith(".java"))
                            {
                                mCommandList.add(item.getAbsolutePath());  // JAVA
                            }
                            else if ((lowerCaseName.endsWith(".ctxt")) || (lowerCaseName.endsWith(".class")))
                            {
                                if (item.canWrite()) item.delete();   // delete binaries
                            }
                        }
                    }
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_COMPILEPACKAGE);
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING_PACKAGE + " " + mPackage.getName() + " ...\n");
                    viewer.setVisible(true);
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    viewer.getOutErrStreamPanel().getOutTA().append(((0 == viewer.GetProcessExitValue()) ? MSG_COMPILATION_COMPLETED : MSG_COMPILATION_FAILED) + "\n");
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                }
                catch (IOException ioEx) {}
                //catch (InterruptedException irEx) {}
                catch (ProjectNotOpenException pnoEx) {}
                catch (PackageNotFoundException pnfEx) {}
            }
        }

        class NXTCompileAction extends XAction
        {
            public NXTCompileAction()
            {
                super(NXT_COMPILE_ACTION,MSG_NXT_COMPILE);
                addLock(mCompileLock);
            }

            public void execute()
            {
                try
                {
                    mCommandList.clear();
                    mCommandList.addAll(mCompileCommandListNXT);
                    if (mWindows)
                    {
                        mCommandList.add("\"" + mProject.getDir().getAbsolutePath() + "\"");     // SOURCEPATH
                        mCommandList.add("\"" + mClass.getJavaFile().getAbsolutePath() + "\"");  // JAVAFILE
                    }
                    else
                    {
                        mCommandList.add(mProject.getDir().getAbsolutePath());
                        mCommandList.add(mClass.getJavaFile().getAbsolutePath());
                    }
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_COMPILE);
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING + " " + mClass.getName() + " ...\n");
                    viewer.setVisible(true);
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    viewer.getOutErrStreamPanel().getOutTA().append(((0 == viewer.GetProcessExitValue()) ? MSG_COMPILATION_COMPLETED : MSG_COMPILATION_FAILED) + "\n");
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                }
                catch (IOException ioEx) {}
                //catch (InterruptedException irEx) {}
                catch (ProjectNotOpenException pnoEx) {}
                catch (PackageNotFoundException pnfEx) {}
            }
        }

        class NXTUploadAction extends XAction
        {
            public NXTUploadAction()
            {
                super(NXT_UPLOAD_ACTION,MSG_NXT_UPLOAD);
                addLock(mConnectionLock);
                addLock(mCompileLock);
            }

            public void execute()
            {
                try
                {
                    mCommandList.clear();
                    mCommandList.addAll(mCompileCommandListNXT);
                    if (mWindows)
                    {
                        mCommandList.add("\"" + mProject.getDir().getAbsolutePath() + "\"");     // SOURCEPATH
                        mCommandList.add("\"" + mClass.getJavaFile().getAbsolutePath() + "\"");  // JAVAFILE
                    }
                    else
                    {
                        mCommandList.add(mProject.getDir().getAbsolutePath());     // SOURCEPATH
                        mCommandList.add(mClass.getJavaFile().getAbsolutePath());  // JAVAFILE
                    }
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_UPLOAD);
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING + " " + mClass.getName() + " ...\n");
                    viewer.setVisible(true);
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    if (0 == viewer.GetProcessExitValue())
                    {
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_COMPLETED + "\n\n");
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_NXT_UPLOAD + " ...\n\n");
                    }
                    else
                    {
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_FAILED + "\n");
                        viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                        return;
                    }
                    mCommandList.clear();
                    mCommandList.addAll(mLinkAndUploadCommandListNXT);
                    if (mWindows)
                    {
                       mCommandList.add((null != mClasspathUserlibNXJ) ? (mClasspathUserlibNXJ + File.pathSeparator + "\"" + mProject.getDir().getAbsolutePath() + "\"") : "\"" + mProject.getDir().getAbsolutePath() + "\""); // CLASSPATH
                       mCommandList.add(mClass.getName());
                    }
                    else
                    {
                       mCommandList.add((null != mClasspathUserlibNXJ) ? (mClasspathUserlibNXJ + File.pathSeparator + mProject.getDir().getAbsolutePath()) : mProject.getDir().getAbsolutePath()); // CLASSPATH
                       mCommandList.add(mClass.getName());
                    }
                    p = mPB.start();
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                    //p.waitFor();
                }
                catch (IOException ioEx) {}
                //catch (InterruptedException irEx) {}
                catch (ProjectNotOpenException pnoEx) {}
                catch (PackageNotFoundException pnfEx) {}
            }
        }

        class EV3SelectRobotAction extends XAction
        {
            public EV3SelectRobotAction()
            {
                super(EV3_SELECT_ROBOT_ACTION,MSG_EV3_ROBOT);
                addLock(mConnectionLock);
            }

            public void execute()
            {
                String robot_id = JOptionPane.showInputDialog(MSG_EV3_ROBOT_ADDRESS,mBlueJ.getExtensionPropertyString("org.jabotics.robot.id","10.0.1.1"));
                if ( (null != robot_id) && (robot_id.length() > 0) )
                {
                    mBlueJ.setExtensionPropertyString("org.jabotics.robot.id",robot_id);
                }
            }
        }

        class EV3ControlAction extends XAction
        {
            public EV3ControlAction()
            {
                super(EV3_CONTROL_ACTION,MSG_EV3_CONTROL);
                addLock(mConnectionLock);
            }

            public void execute()
            {
                try
                {
                    mCommandList.clear();
                    mCommandList.addAll(mControlCommandListEV3);
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_CONTROL);
                    viewer.showOnError(true);
                    viewer.automaticTermination(true);
                    viewer.startLogging(p);
                    p.waitFor();
                }
                catch (IOException ioEx) {}
                catch (InterruptedException irEx) {}
            }
        }

        class EV3SDCardAction extends XAction
        {
            public EV3SDCardAction()
            {
                super(EV3_SDCARD_ACTION,MSG_EV3_SDCARD);
                addLock(mConnectionLock);
            }

            public void execute()
            {
                try
                {
                    mCommandList.clear();
                    mCommandList.addAll(mSDCardCommandListEV3);
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_SDCARD);
                    viewer.showOnError(true);
                    viewer.automaticTermination(true);
                    viewer.startLogging(p);
                    p.waitFor();
                }
                catch (IOException ioEx) {}
                catch (InterruptedException irEx) {}
            }
        }


        class EV3UserlibsUploadAction extends XAction
        {
            public EV3UserlibsUploadAction()
            {
                super(EV3_LIBS_UPLOAD_ACTION,MSG_EV3_LIBS);
                addLock(mConnectionLock);
            }

            public void execute()
            {
                if (mFilesUserlibEV3.isEmpty())
                {
                    System.err.println(MSG_EV3_LIBS_FAILED);
                    return;
                }

                String robot_id = mBlueJ.getExtensionPropertyString("org.jabotics.robot.id","10.0.1.1");

                XProcessViewer viewer = new XProcessViewer();
                viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_LIBS);
                viewer.getOutErrStreamPanel().getOutTA().append(MSG_EV3_LIBS + " -> " + robot_id + " ...\n");
                viewer.setVisible(true);
                for (File file : mFilesUserlibEV3)
                {
                    try
                    {
                        mCommandList.clear();
                        mCommandList.addAll(mLibUploadCommandListEV3);
                        mCommandList.add(robot_id);
                        mCommandList.add(file.getCanonicalPath());
                        mCommandList.add("/home/lejos/lib/"+file.getName());
                        viewer.getOutErrStreamPanel().getOutTA().append(file.getName() + " ... ");
                        Process p = mPB.start();
                        viewer.startLogging(p);
                        if (!viewer.waitFor())
                        {
                            // Viewer vorzeitig beendet
                            p.destroy();
                            viewer.dispose();
                            return;
                        }
                        viewer.getOutErrStreamPanel().getOutTA().append((0 == viewer.GetProcessExitValue()) ? MSG_EV3_UPLOAD_COMPLETED + "\n" : MSG_EV3_UPLOAD_FAILED + "\n");
                    }
                    catch (IOException ioEx) {}
                }
                viewer.getOutErrStreamPanel().getOutTA().append(MSG_EV3_LIBS_COMPLETED + "\n");
                viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            }
        }

        class EV3CompilePackageAction extends XAction
        {
            public EV3CompilePackageAction()
            {
                super(EV3_COMPILEPACKAGE_ACTION,MSG_EV3_COMPILEPACKAGE);
                addLock(mCompileLock);
            }
            
            public void execute()
            {
                try
                {
                    mCommandList.clear();
                    mCommandList.addAll(mCompileCommandListEV3);
                    mCommandList.add(mProject.getDir().getAbsolutePath());  // SOURCEPATH
                    File[] files = mPackage.getDir().listFiles();  // look for java files in package directory
                    for (File item : files)
                    {
                        if (item.isFile())
                        {
                            String lowerCaseName = item.getName().toLowerCase();
                            if (lowerCaseName.endsWith(".java"))
                            {
                                mCommandList.add(item.getAbsolutePath());  // JAVA
                            }
                            else if ((lowerCaseName.endsWith(".ctxt")) || (lowerCaseName.endsWith(".class")))
                            {
                                if (item.canWrite()) item.delete();   // delete binaries
                            }
                        }
                    }
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_COMPILEPACKAGE);
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING_PACKAGE + " " + mPackage.getName() + " ...\n");
                    viewer.setVisible(true);
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    viewer.getOutErrStreamPanel().getOutTA().append(((0 == viewer.GetProcessExitValue()) ? MSG_COMPILATION_COMPLETED : MSG_COMPILATION_FAILED) + "\n");
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                }
                catch (IOException ioEx) {}
                //catch (InterruptedException irEx) {}
                catch (ProjectNotOpenException pnoEx) {}
                catch (PackageNotFoundException pnfEx) {}
            }
        }

        class EV3CompileAction extends XAction
        {
            public EV3CompileAction()
            {
                super(EV3_COMPILE_ACTION,MSG_EV3_COMPILE);
                addLock(mCompileLock);
            }

            public void execute()
            {
                try
                {
                    String fileName = mClass.getJavaFile().getAbsolutePath();
                    fileName = fileName.substring(0,fileName.length()-5) + ".jar"; // jar file to create

                    mCommandList.clear();
                    mCommandList.addAll(mCompileCommandListEV3);
                    if (mWindows)
                    {
                        mCommandList.add("\"" + mProject.getDir().getAbsolutePath() + "\"");     // SOURCEPATH
                        mCommandList.add("\"" + mClass.getJavaFile().getAbsolutePath() + "\"");  // JAVAFILE
                    }
                    else
                    {
                        mCommandList.add(mProject.getDir().getAbsolutePath());
                        mCommandList.add(mClass.getJavaFile().getAbsolutePath());
                    }
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_COMPILE);
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING + " " + mClass.getName() + " ...\n");
                    viewer.setVisible(true);
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    if (0 == viewer.GetProcessExitValue())
                    {
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_COMPLETED + "\n\n");
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_JAR + " ...\n");
                    }
                    else
                    {
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_FAILED + "\n");
                        viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                        return;
                    }
                    mCommandList.clear();
                    mCommandList.addAll(mJarCommandListEV3);
                    mCommandList.add(fileName);
                    mCommandList.add(mClass.getName()); // class to execute
                    mCommandList.add("-C"); // change base directory
                    mCommandList.add(mProject.getDir().getAbsolutePath());
                    mCommandList.add(mClass.getName().replace('.',File.separatorChar) + ".class");
                    p = mPB.start();
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    viewer.getOutErrStreamPanel().getOutTA().append((0 == viewer.GetProcessExitValue()) ? MSG_JAR_COMPLETED + "\n" : MSG_JAR_FAILED + "\n");
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                }
                catch (IOException ioEx) {}
                //catch (InterruptedException irEx) {}
                catch (ProjectNotOpenException pnoEx) {}
                catch (PackageNotFoundException pnfEx) {}
            }
        }

        class EV3UploadAction extends XAction
        {
            public EV3UploadAction()
            {
                super(EV3_UPLOAD_ACTION,MSG_EV3_UPLOAD);
                addLock(mConnectionLock);
                addLock(mCompileLock);
            }

            public void execute()
            {
                try
                {
                    String robot_id = mBlueJ.getExtensionPropertyString("org.jabotics.robot.id","10.0.1.1");
                    String fileName = mClass.getJavaFile().getAbsolutePath();
                    fileName = fileName.substring(0,fileName.length()-5) + ".jar"; // jar file to create

                    mCommandList.clear();
                    mCommandList.addAll(mCompileCommandListEV3);
                    if (mWindows)
                    {
                        mCommandList.add("\"" + mProject.getDir().getAbsolutePath() + "\"");     // SOURCEPATH
                        mCommandList.add("\"" + mClass.getJavaFile().getAbsolutePath() + "\"");  // JAVAFILE
                    }
                    else
                    {
                        mCommandList.add(mProject.getDir().getAbsolutePath());     // SOURCEPATH
                        mCommandList.add(mClass.getJavaFile().getAbsolutePath());  // JAVAFILE
                    }
                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_UPLOAD);
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING + " " + mClass.getName() + " ...\n");
                    viewer.setVisible(true);
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    if (0 == viewer.GetProcessExitValue())
                    {
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_COMPLETED + "\n\n");
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_JAR + " ...\n");
                    }
                    else
                    {
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_FAILED + "\n");
                        viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                        return;
                    }
                    mCommandList.clear();
                    mCommandList.addAll(mJarCommandListEV3);
                    mCommandList.add(fileName);
                    mCommandList.add(mClass.getName()); // class to execute
                    mCommandList.add("-C"); // change base directory
                    mCommandList.add(mProject.getDir().getAbsolutePath());
                    mCommandList.add(mClass.getName().replace('.',File.separatorChar) + ".class");
                    p = mPB.start();
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    if (0 == viewer.GetProcessExitValue())
                    {
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_JAR_COMPLETED + "\n\n");
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_EV3_UPLOAD + " -> " + robot_id + " ...\n");
                    }
                    else
                    {
                        viewer.getOutErrStreamPanel().getOutTA().append(MSG_JAR_FAILED + "\n");
                        viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                        return;
                    }
                    mCommandList.clear();
                    mCommandList.addAll(mUploadCommandListEV3);
                    mCommandList.add(robot_id);
                    mCommandList.add(fileName);
                    fileName = "/home/lejos/programs/" + fileName.substring(fileName.lastIndexOf(File.separatorChar));
                    mCommandList.add(fileName);
                    p = mPB.start();
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    viewer.getOutErrStreamPanel().getOutTA().append((0 == viewer.GetProcessExitValue()) ? MSG_EV3_UPLOAD_COMPLETED + "\n" : MSG_EV3_UPLOAD_FAILED + "\n");
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                    //p.waitFor();
                }
                catch (IOException ioEx) {}
                //catch (InterruptedException irEx) {}
                catch (ProjectNotOpenException pnoEx) {}
                catch (PackageNotFoundException pnfEx) {}
            }
        }

        class SimulateAction extends XAction
        {
            public SimulateAction()
            {
                super(SIMULATE_ACTION,MSG_SIM_START);
                addLock(mCompileLock);
            }

            public void execute()
            {
                try
                {
                    mClass.compile(true);
                    if (!mClass.isCompiled()) return;

                    mCommandList.clear();
                    mCommandList.addAll(mSimulateCommandList);
                    if (mWindows)
                    {
                       mCommandList.add((null != mClasspathUserlibBlueJ) ? (mClasspathUserlibBlueJ + File.pathSeparator + "\"" + mProject.getDir().getAbsolutePath() + "\"") : "\"" + mProject.getDir().getAbsolutePath() + "\""); // CLASSPATH
                    }
                    else
                    {
                       mCommandList.add((null != mClasspathUserlibBlueJ) ? (mClasspathUserlibBlueJ + File.pathSeparator + mProject.getDir().getAbsolutePath()) : mProject.getDir().getAbsolutePath()); // CLASSPATH
                    }
                    mCommandList.add("org.jabotics.robot.sim.XSimulation");
                    mCommandList.add("-l");
                    mCommandList.add(mLanguage);
                    mCommandList.add(mClass.getName());

                    Process p = mPB.start();
                    XProcessViewer viewer = new XProcessViewer();
                    viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_SIM_START);
                    viewer.showOnError(true);
                    viewer.startLogging(p);
                    viewer.waitFor();
                    if (viewer.isVisible())
                    {
                        viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                    }
                    else
                    {
                        p.destroy();
                        viewer.dispose();
                    }
                }
                catch (CompilationNotStartedException cnsEx) {}
                catch (IOException ioEx) {}
                catch (ProjectNotOpenException pnoEx) {}
                catch (PackageNotFoundException pnfEx) {}
            }
        }
    }


    static public void test()
    {
        XBlueJExtension extension = new XBlueJExtension();
        extension.mLanguage = "de";
        extension.initMessages();
    }

    /*static public void test()
    {
        try
        {
            String robot_id = mBlueJ.getExtensionPropertyString("org.jabotics.robot.id","10.0.1.1");
    
            mCommandList.clear();
            mCommandList.addAll(mLibUploadCommandListEV3);
            mCommandList.add(robot_id);
            mCommandList.add(mJaboticsEV3);
            mCommandList.add("/home/lejos/lib/jabotics-ev3.jar");
            Process p = mPB.start();
            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_LIBS);
            viewer.getOutErrStreamPanel().getOutTA().append(MSG_EV3_LIBS + " -> " + robot_id + " ...\n");
            viewer.setVisible(true);
            viewer.startLogging(p);
            if (!viewer.waitFor())
            {
                // Viewer vorzeitig beendet
                p.destroy();
                viewer.dispose();
                return;
            }
            viewer.getOutErrStreamPanel().getOutTA().append((0 == viewer.GetProcessExitValue()) ? MSG_EV3_LIBS_COMPLETED + "\n" : MSG_EV3_LIBS_FAILED + "\n");
            viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        }
        catch (IOException ioEx) {}
    }*/

    /*static public void testUbuntu()
    {
        try
        {
            Runtime r = Runtime.getRuntime();
            //Process p = r.exec("java -classpath D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterBlueJErweiterung org.jabotics.nxt.bluej.XTestprozess");
            //Process p = r.exec("java -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJFlashG");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" -implicit:class -sourcepath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java\"");
            Process p = r.exec("java -Dnxj.home=\"/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3\" -DCOMMAND_NAME=nxjlink -classpath \"/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/pc/pctools.jar\" lejos.pc.tools.NXJLink --writeorder \"LE\" --bootclasspath \"/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/nxt/classes.jar\" --extdirs \"/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/userlib\" --classpath \"/media/Daten/Programmierung/BlueJ/StandardRoboterTest\" --\"/media/Daten/Programmierung/BlueJ/StandardRoboterTest/XRobot.class\"");
            // java -Dnxj.home="/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3" -DCOMMAND_NAME=nxjlink -classpath "/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/pc/pctools.jar" lejos.pc.tools.NXJLink --writeorder \"LE\" --bootclasspath "/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/nxt/classes.jar" --extdirs "/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/userlib" --classpath "/media/Daten/Dokumente/Programmierung/BlueJ/StandardRoboterTest" --"/media/Daten/Dokumente/Programmierung/BlueJ/StandardRoboterTest/XRobot.class"
       
            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle("NXJ-Erweiterung");
            viewer.setVisible(true);
            viewer.getOutErrStreamPanel().getOutTA().append("Ausgabe des Prozesses:\n");
            viewer.getErrTA().append("Fehlermeldungen:\n");
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            try
            {
                Thread.sleep(30000);
            }
            catch(InterruptedException e) {}
        }
        catch (IOException ex) {}
        catch (InterruptedException ex) {}
        System.exit(0);
    }*/

    /*public void test2Ubuntu()
    {
        //startup();

        try
        {
            //Process p = r.exec("java -classpath D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterBlueJErweiterung org.jabotics.nxt.bluej.XTestprozess");
            //Process p = r.exec("java -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJFlashG");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" -implicit:class -sourcepath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java\"");
            mCompileCommandListNXT.set(mCompileCommandListNXT.size()-2, "/media/Daten/Programmierung/BlueJ/StandardRoboterTest");
            mCompileCommandListNXT.set(mCompileCommandListNXT.size()-1, "/media/Daten/Programmierung/BlueJ/StandardRoboterTest/XRobot.java");
            mLinkAndUploadCommandListNXT.set(mLinkAndUploadCommandListNXT.size()-2, "/media/Daten/Programmierung/BlueJ/StandardRoboterTest");
            mLinkAndUploadCommandListNXT.set(mLinkAndUploadCommandListNXT.size()-1, "XRobot");
            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle("NXJ-Erweiterung");
            viewer.setVisible(true);
            viewer.getOutErrStreamPanel().getOutTA().append("Ausgabe des Prozesses:\n");
            viewer.getErrTA().append("Fehlermeldungen:\n");
            Process p = mPBCompile.start();
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            p = mPBLinkAndUpload.start();
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            Thread.sleep(30000);
            System.exit(0);
        }
        catch (IOException ioEx) {}
        catch(InterruptedException irEx) {}
    }*/

    /*static public void testWin()
    {
        try
        {
            Runtime r = Runtime.getRuntime();
            //Process p = r.exec("java -classpath D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterBlueJErweiterung org.jabotics.nxt.bluej.XTestprozess");
            //Process p = r.exec("java -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJFlashG");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" -implicit:class -sourcepath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java\"");
            Process p = r.exec("java -Dnxj.home=\"C:\\Programme\\lejos NXJ\" -DCOMMAND_NAME=nxjlink -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJLink --writeorder \"LE\" --bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" --extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" --classpath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" --\"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot.class\"");
            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle("NXJ-Erweiterung");
            viewer.setVisible(true);
            .append("Ausgabe des Prozesses:\n");
            viewer.getErrTA().append("Fehlermeldungen:\n");
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            try
            {
                Thread.sleep(30000);
            }
            catch(InterruptedException e) {}
        }
        catch (IOException ex) {}
        catch (InterruptedException ex) {}
        System.exit(0);
    }*/

    /*public void test2Win()
    {
        startup();

        try
        {
            //Process p = r.exec("java -classpath D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterBlueJErweiterung org.jabotics.nxt.bluej.XTestprozess");
            //Process p = r.exec("java -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJFlashG");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" -implicit:class -sourcepath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java\"");
            /*mCompileCommandListNXT.set(mCompileCommandListNXT.size()-2, "\"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\"");
            mCompileCommandListNXT.set(mCompileCommandListNXT.size()-1, "\"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot.java\"");
            mLinkAndUploadCommandListNXT.set(mLinkAndUploadCommandListNXT.size()-2, "\"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\"");
            mLinkAndUploadCommandListNXT.set(mLinkAndUploadCommandListNXT.size()-1, "\"XRobot\"");
            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle("NXJ-Erweiterung");
            viewer.setVisible(true);
            viewer.getOutTA().append("Ausgabe des Prozesses:\n");
            viewer.getErrTA().append("Fehlermeldungen:\n");
            Process p = mPBCompile.start();
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            p = mPBLinkAndUpload.start();
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            Thread.sleep(30000);
            System.exit(0);
        }
        catch (IOException ioEx) {}
        //catch(InterruptedException irEx) {}
    }*/
}
