/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.bluej; 

import org.jabotics.robot.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Use XProcessViewer to log out and error streams for compilation and other processes.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class XProcessViewer extends JFrame implements XLogStreamTextArea.ILogListener
{
    static protected Timer sTimer = new Timer("Timer for XProcessViewer", true);

    /*class XProcessViewerWindowAdapter extends WindowAdapter
    {
        public void windowClosing(WindowEvent e)
        {
            if (e.getComponent() instanceof XProcessViewer)
            {
                XProcessViewer processviewer = (XProcessViewer)e.getComponent();
                processviewer.showOnOutput(false);
                processviewer.showOnError(false);
            }
        }
    }*/

    class XProcessValidationTask extends TimerTask
    {
        // Instanzvariablen
        Process mProcess;
        volatile boolean mBlocked;
 
        XProcessValidationTask(Process p)
        {
            super();
            mProcess = p;
            mBlocked = false;
        }
        
        synchronized boolean aquire()
        {
            if (mBlocked) return false;
            mBlocked = true;
            return true;
        }

        synchronized void release()
        {
            mBlocked = false;
        }

        public void run()
        {
            if (!aquire()) return;  // Notwendig???  Stellt sicher, dass run() nicht parallel ausgeführt wird und weitere Threads nicht auf Ausführung warten.

            try
            {
                int tmp = mProcess.exitValue();
                
                // no exception thrown => process has been terminated
                mProcessExitValue = tmp;
                processTerminated(); // terminates this task, also
            }
            catch(IllegalThreadStateException e) {}
            finally {release();}
        }
    }

    // instance variables
    private XProcessValidationTask mProcessValidationTask = null;
    private int mProcessExitValue = 0;
    private boolean mAutoTerminate = false;
    private boolean mProcessCompleted = true;
    private XOutErrStreamPanel mOutErrPanel;

    /**
     * Konstruktor für Objekte der Klasse XProcessViewer
     */
    public XProcessViewer()
    {
        super();
        mOutErrPanel = new XOutErrStreamPanel();
        mOutErrPanel.setPreferredSize(new Dimension(760,520));
        this.add(mOutErrPanel);
        this.pack();
        mOutErrPanel.getSplitPane().setDividerLocation(1.0);
        //this.addWindowListener(new XProcessViewerWindowAdapter());
    }

    public XOutErrStreamPanel getOutErrStreamPanel()
    {
        return mOutErrPanel;
    }

    public void automaticTermination(boolean autoterminate)
    {
        mAutoTerminate = autoterminate;
    }

    public void showOnOutput(boolean show)
    {
        if (show)
        {
            mOutErrPanel.getOutTA().addListener(this);
        }
        else
        {
            mOutErrPanel.getOutTA().removeListener(this);
        }
    }

    public void showOnError(boolean show)
    {
        if (show)
        {
            mOutErrPanel.getErrTA().addListener(this);
        }
        else
        {
            mOutErrPanel.getErrTA().removeListener(this);
        }
    }

    public void logEvent(XLogStreamTextArea source)
    {
        if (!this.isVisible()) this.setVisible(true);
    }

    public void startLogging(Process process)
    {
        stopLogging();

        mOutErrPanel.startLogging(process.getInputStream(),process.getErrorStream());

        mProcessValidationTask = new XProcessValidationTask(process);
        mProcessCompleted = false;
        mProcessExitValue = 0;
        sTimer.schedule(mProcessValidationTask, 0, 100);
    }

    synchronized public void stopLogging()
    {
        if (null == mProcessValidationTask) return;
        
        mOutErrPanel.stopLogging();

        mProcessValidationTask.cancel();    // mark task as completed
        sTimer.purge();                     // remove task from timers task list
        mProcessValidationTask = null;      // final cleanup of task
    }

    private void processTerminated()
    {
        mProcessCompleted = true;
        this.stopLogging();
        if (mAutoTerminate && !this.isVisible()) this.dispose(); // Falls sichtbar, wird Viewer von Anwender beendet
    }

    public boolean waitFor()
    {
        while (true)
        {
            try
            {
                if (null == mProcessValidationTask) return mProcessCompleted;
                Thread.sleep(50);
            }
            catch (InterruptedException e) {}
        }
    }
    
    public int GetProcessExitValue()
    {
        return mProcessExitValue;
    }
}
    