/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;


/**
 * <font color="LimeGreen"><b>&bull;</b></font> Die Schnittstelle ITastsensor erweitert
 * die Schnittstelle {@link ISensor} und bietet vereinfachte Zugriffsmethoden für alle Tastsensoren.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public interface ITastsensor extends ISensor
{ 
    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Liefert den Status des Tastsensors.
     * 
     * @return  liefert den Wert true, falls gedrückt, und andernfalls den Wert false
     */
    public boolean istGedrueckt();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung,
     * bis sich der Tastsensor im Zustand <i>gedrückt</i> befindet.
     */
    public void warteBisGedrueckt();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung,
     * bis sich der Tastsensor im Zustand <i>losgelassen</i> befindet.
     */
    public void warteBisLosgelassen();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung,
     * bis sich der Tastsensor im Zustand <i>gedrückt</i> befindet oder die maximale Wartezeit
     * überschritten wird.
     * 
     * @param   maximale_wartezeit     die maximale Wartezeit, in der auf das Drücken des Tasters gewartet wird
     * 
     * @return  true, falls der Sensor gedrückt wurde bzw. false, falls die maximale Wartezeit überschritten wurde
     */
    public boolean warteBisGedrueckt(int maximale_wartezeit);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung,
     * bis sich der Tastsensor im Zustand <i>losgelassen</i> befindet oder die maximale Wartezeit
     * überschritten wird.
     * 
     * @param maximale_wartezeit     die maximale Wartezeit, in der auf das Loslassen des Tasters gewartet wird
     * 
     * @return  true, falls der Sensor losgelassen wurde bzw. false, falls die maximale Wartezeit überschritten wurde
     */
    public boolean warteBisLosgelassen(int maximale_wartezeit);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung,
     * bis der Tastsensor gedrückt wird, d.h. bis sich sein Zustand von <i>losgelassen</i> nach
     * <i>gedrückt</i> ändert.

     */
    public void warteBisEreignisGedrueckt();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung,
     * bis der Sensor losgelassen wird, d.h. bis sich sein Zustand von <i>gedrückt</i> nach
     * <i>losgelassen</i> ändert.
     */
    public void warteBisEreignisLosgelassen();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung,
     * bis der Tastsensor gedrückt wird, d.h. bis sich sein Zustand von <i>losgelassen</i> nach
     * <i>gedrückt</i> ändert, oder die maximale Wartezeit überschritten wird.
     * 
     * @param   maximale_wartezeit     die maximale Wartezeit, in der auf das Drücken des Tasters gewartet wird
     * 
     * @return  true, falls der Sensor gedrückt wurde bzw. false, falls die maximale Wartezeit überschritten wurde
     */
    public boolean warteBisEreignisGedrueckt(int maximale_wartezeit);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Unterbricht die weitere Programmausführung,
     * bis der Sensor losgelassen wird, d.h. bis sich sein Zustand von <i>gedrückt</i> nach
     * <i>losgelassen</i> ändert oder die maximale Wartezeit überschritten wird.
     * 
     * @param   maximale_wartezeit     die maximale Wartezeit, in der auf das Loslassen des Tasters gewartet wird
     * 
     * @return  true, falls der Sensor losgelassen wurde bzw. false, falls die maximale Wartezeit überschritten wurde
     */
    public boolean warteBisEreignisLosgelassen(int maximale_wartezeit);
}
