/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;

import org.jabotics.robot.defs.*;
import org.jabotics.robot.core.CMotor;

/**
 * Objekte vom Typ XMotor repräsentieren einen Motor des NXT-Roboters.
 * Die Klasse XMotor stellt dem Nutzer die Schnittstelle
 * {@link IMotor} zur Ansteuerung zur Verfügung.
 * Der Konstruktor der Klasse ist "nicht öffentlich". Man kann deshalb
 * als Anwender selbst keine Objekte dieses Typs erzeugen.
 * Man verschafft sich Zugang zu einem neuen Motor-Objekt, indem man die Methode
 * {@link IObjektfabrik#erzeugeMotor()} aufruft.
 * Man erhält Zugriff auf die einzige Objektfabrik-Instanz, indem man
 * die statische Methode {@link XObjektfabrik#holeObjektfabrik()} aufruft.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
class XMotor implements IMotor, CMotor.ICaller
{
    // Instanzvariablen
    private CMotor mMotor;

    XMotor(EMotoranschluss anschluss)
    {
        mMotor = CMotor.getMotor(EMotorPort.values()[anschluss.ordinal()]);
    }

    public void stopControl(Object ressource)
    {
        // ressource = Instanz von CMotorController
        // keine Aktion erforderlich
    }

    public void setzeKorrekturfaktorWinkelstand(float korrekturfaktor)
    {
        mMotor.setTachoCountCorrectionFactor(korrekturfaktor);
    }

    public float holeWinkelstand()
    {
        return mMotor.getPosition(this);
    }

    public void setzeWinkelstandZurueck()
    {
        mMotor.resetPosition(this);
        CMotor.execute(this,mMotor,false);
    }
    
    public float holeGeschwindigkeit()
    {
        return mMotor.getSpeed();
    }
    
    public float holeSollgeschwindigkeit()
    {
        return mMotor.getTargetSpeed(this);
    }
    
    public void setzeSollgeschwindigkeit(float winkelgeschwindigkeit)
    {
        mMotor.setTargetSpeed(this, winkelgeschwindigkeit);
        CMotor.execute(this,mMotor,false);
    }
    
    public float holeSollbeschleunigung()
    {
        return mMotor.getTargetAcceleration(this);
    }
    
    public void setzeSollbeschleunigung(float winkelbeschleunigung)
    {
        mMotor.setTargetAcceleration(this, winkelbeschleunigung);
        CMotor.execute(this,mMotor,false);
    }

    public float holeMaximalgeschwindigkeit()
    {
        return mMotor.getMaxSpeed();
    }
    
    public void setzeSollgeschwindigkeitMaximal()
    {
        mMotor.setTargetSpeedToMax(this);
        CMotor.execute(this,mMotor,false);
    }
    
    public float holeMaximalbeschleunigung()
    {
        return mMotor.getMaxAcceleration();
    }
    
    public void setzeSollbeschleunigungMaximal()
    {
        mMotor.setTargetAccelerationToMax(this);
        CMotor.execute(this,mMotor,false);
    }
    
    public void drehe(ERichtung richtung)
    {
        mMotor.startRotationUnlimited(this, (ERichtung.VORWAERTS == richtung));
        CMotor.execute(this,mMotor,false);
    }
    
    public void dreheZeit(float dauer, ERichtung richtung, EWartemodus wartemodus)
    {
        mMotor.startRotationTimeLimited(this, dauer, (ERichtung.VORWAERTS == richtung));
        CMotor.execute(this,mMotor,(EWartemodus.ABWARTEN == wartemodus));
    }

    public void dreheUmWinkel(float drehwinkel, EWartemodus wartemodus)
    {
        mMotor.startRotationPosLimited(this, drehwinkel, true);
        CMotor.execute(this,mMotor,(EWartemodus.ABWARTEN == wartemodus));
    }

    public void dreheZuWinkelstand(float winkelstand, EWartemodus wartemodus)
    {
        mMotor.startRotationPosLimited(this, winkelstand, false);
        CMotor.execute(this,mMotor,(EWartemodus.ABWARTEN == wartemodus));
    }

    public void warteEndeAb()
    {
        CMotor.waitUntilCompleted(this,mMotor);
    }
    
    public void halteAn(EWartemodus wartemodus)
    {
        mMotor.stopRotation(this);
        CMotor.execute(this,mMotor,(EWartemodus.ABWARTEN == wartemodus));
    }
    
    public void gebeFrei()
    {
        mMotor.setToFloatMode(this);
        CMotor.execute(this,mMotor,false);
    }
    
    public boolean istInBewegung()
    {
        return mMotor.isMoving();
    }
    
    public boolean istAusgesetzt()
    {
        return mMotor.isStalled();
    }
}
