/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;

import org.jabotics.robot.defs.ESensorType;

/**
 * <font color="Gold"><b>&bull;</b></font> Standard-Implementierung
 * der zentralen Schnittstelle IObjektfabrik, die zur Erzeugung
 * von Instanzen anderer Klassen der Roboterbibliothek benötigt wird.
 * Der Konstruktor dieser Klasse ist "privat". Man kann deshalb
 * außerhalb der Klasse selbst kein Objekt vom Typ XObjektfabrik erzeugen.
 * Man verschafft sich Zugang zur einzigen Objektfabrik-Instanz,
 * die von der Klasse XObjektfabrik selbst erzeugt und verwaltet wird,
 * indem man die statische Methode {@link #holeObjektfabrik()} aufruft.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
class XObjektfabrik implements IObjektfabrik
{
    XObjektfabrik()
    {
    }
    
    public IMotor erzeugeMotor(EMotoranschluss anschluss)
    {
        return new XMotor(anschluss);
    }
    
    public IPilot erzeugePiloten(EMotoranschluss anschlussLinkerMotor, EMotoranschluss anschlussRechterMotor)
    {
        return new XPilot(anschlussLinkerMotor, anschlussRechterMotor);
    }
    
    public IPilot erzeugePiloten(EMotoranschluss anschlussLinkerMotor, EMotoranschluss anschlussRechterMotor, float raddurchmesser, float radabstand)
    {
        return new XPilot(anschlussLinkerMotor, anschlussRechterMotor, raddurchmesser, radabstand);
    }

    public ITastsensor erzeugeTastsensor(ETaster taster)
    {
        return new XTastsensor(taster);
    }
    
    public ITastsensor erzeugeTastsensor(ESensoranschluss anschluss)
    {
        return new XTastsensor(anschluss);
    }
    
    public ISensor erzeugeUltraschallsensor(ESensoranschluss anschluss)
    {
        return new XSensor(anschluss,ESensorType.ULTRASONIC_SENSOR);
    }
    
    public ISensor erzeugeSchallsensor(ESensoranschluss anschluss)
    {
        return new XSensor(anschluss,ESensorType.SOUND_SENSOR);
    }
    
    public ILichtsensor erzeugeLichtsensor(ESensoranschluss anschluss)
    {
        return new XLichtsensor(anschluss);
    }
    
    public IMonitor erzeugeMonitor()
    {
        return new XMonitor();
    }
    
    public ILautsprecher erzeugeLautsprecher()
    {
        return new XLautsprecher();
    }
    
    public IUhr erzeugeUhr()
    {
        return new XUhr();
    }
    
    public IPCKonsole erzeugePCKonsole()
    {
        return new XPCKonsole();
    }
}
