/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;


/**
 * <font color="LimeGreen"><b>&bull;</b></font> The interface IMonitor provides methods
 * for outputs to the LCD display of the robot brick. Note that positions an the display
 * are specified by a position within a line (x-position) and a line number (y-position)
 * both starting with 0 and going from left to right and top to buttom, respectively.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */

public interface IDisplay
{
    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Initiates an update of the display.
     */
    public void refresh();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Clears the complete display.
     */
    public void clear();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Clears a single line on the display.
     * 
     * @param  y_position   line number for the line to clear
     */
    public void clearLine(int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Clears a row of characters on the display.
     * 
     * @param  x_position   x-position of the first charcter to be cleared
     * @param  y_position   line number of the characters to be cleared
     * @param  number       number of characters to be cleared
     */
    public void clearCharacters(int x_position, int y_position, int number);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Writes a single character to the display.
     * 
     * @param  character    character to be written to the display
     * @param  x_position   x-position on the display
     * @param  y_position   line number on the display
     */
    public void write(char character, int x_position, int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Writes an integer value to the display.
     * 
     * @param  integral_number  integral number to be written to the display
     * @param  x_position       x-position on the display
     * @param  y_position       line number on the display
     */
    public void write(int integral_number, int x_position, int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Writes an integer value to the display with given number of digits.
     * 
     * @param  integral_number  integral number to be written to the display
     * @param  x_position       x-position on the display
     * @param  y_position       line number on the display
     * @param  digits           number of digits displayed on the display including leading spaces
     */
    public void write(int integral_number, int x_position, int y_position, int digits);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Writes a decimal value to the display.
     * 
     * @param  decimal_number   decimal number to be written to the display
     * @param  x_position       x-position on the display
     * @param  y_position       line number on the display
     */
    public void write(float decimal_number, int x_position, int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Writes a text to the display.
     * 
     * @param  text         text to be written to the display
     * @param  x_position   x-position on the display
     * @param  y_position   line number on the display
     */
    public void write(String text, int x_position, int y_position);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Writes a text to the display.
     * 
     * @param  text         text to be written to the display
     * @param  x_position   x-position on the display
     * @param  y_position   line number on the display
     * @param  highlighted  text will be displayed highlighted if value is "true"
     */
    public void write(String text, int x_position, int y_position, boolean highlighted);
}
