/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;

import org.jabotics.robot.defs.*;
import org.jabotics.robot.core.CMotor;

/**
 * Objects of type XMotor represent a motor of the robot.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
class XMotor implements IMotor, CMotor.ICaller
{
    // Instanzvariablen
    private CMotor mMotor;

    XMotor(org.jabotics.robot.en.EMotorPort port)
    {
        mMotor = CMotor.getMotor(org.jabotics.robot.defs.EMotorPort.values()[port.ordinal()]);
    }

    public void stopControl(Object ressource)
    {
        // ressource = Instanz von CMotorController
        // keine Aktion erforderlich
    }

    public void setCorrectionFactor(float correctionFactor)
    {
        mMotor.setTachoCountCorrectionFactor(correctionFactor);
    }

    public float getPosition()
    {
        return Math.round(mMotor.getPosition(this));
    }

    public void resetPosition()
    {
        mMotor.resetPosition(this);
        CMotor.execute(this,mMotor,false);
    }
    
    public float getSpeed()
    {
        return mMotor.getSpeed();
    }
    
    public float getTargetSpeed()
    {
        return mMotor.getTargetSpeed(this);
    }
    
    public void setTargetSpeed(float speed)
    {
        mMotor.setTargetSpeed(this, speed);
        CMotor.execute(this,mMotor,false);
    }
    
    public float getTargetAcceleration()
    {
        return mMotor.getTargetAcceleration(this);
    }
    
    public void setTargetAcceleration(float acceleration)
    {
        mMotor.setTargetAcceleration(this, acceleration);
        CMotor.execute(this,mMotor,false);
    }

    public float getMaxSpeed()
    {
        return mMotor.getMaxSpeed();
    }
    
    public void setTargetSpeedToMax()
    {
        mMotor.setTargetSpeedToMax(this);
        CMotor.execute(this,mMotor,false);
    }
    
    public float getMaxAcceleration()
    {
        return mMotor.getMaxAcceleration();
    }

    public void setTargetAccelerationToMax()
    {
        mMotor.setTargetAccelerationToMax(this);
        CMotor.execute(this,mMotor,false);
    }

    public void rotate(EDirection direction)
    {
        mMotor.startRotationUnlimited(this, (EDirection.FORWARD == direction));
        CMotor.execute(this,mMotor,false);
    }

    public void rotateTime(float period, EDirection direction, EWaitingMode waitingMode)
    {
        mMotor.startRotationTimeLimited(this, period, (EDirection.FORWARD == direction));
        CMotor.execute(this,mMotor,(EWaitingMode.WAIT == waitingMode));
    }

    public void rotateByAngle(float angle, EWaitingMode waitingMode)
    {
        mMotor.startRotationPosLimited(this, angle, true);
        CMotor.execute(this,mMotor,(EWaitingMode.WAIT == waitingMode));
    }

    public void rotateToPostion(float position, EWaitingMode waitingMode)
    {
        mMotor.startRotationPosLimited(this, position, false);
        CMotor.execute(this,mMotor,(EWaitingMode.WAIT == waitingMode));
    }

    public void waitUntilRotationCompleted()
    {
        CMotor.waitUntilCompleted(this,mMotor);
    }
    
    public void stopRotation(EWaitingMode waitingMode)
    {
        mMotor.stopRotation(this);
        CMotor.execute(this,mMotor,(EWaitingMode.WAIT == waitingMode));
    }
    
    public void releaseControl()
    {
        mMotor.setToFloatMode(this);
        CMotor.execute(this,mMotor,false);
    }
    
    public boolean isMoving()
    {
        return mMotor.isMoving();
    }
    
    public boolean isStalled()
    {
        return mMotor.isStalled();
    }
}
