/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;

import org.jabotics.robot.defs.ISoundAdapter;
import org.jabotics.robot.adapter.XAdapter;

/**
 * Objects of type XSpeaker represent the speaker of the robot brick.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
class XSpeaker implements ISpeaker
{
    ISoundAdapter mSound = XAdapter.getFactory().getSoundAdapter();

    protected XSpeaker()
    {
    }

    public int getVolume()
    {
        return mSound.getVolume();
    }

    public void setVolume(int volume)
    {
        mSound.setVolume(volume);
    }

    public void playBeep()
    {
        mSound.beep();
    }

    public void playTwoBeeps()
    {
        mSound.twoBeeps();
    }

    public void playBeepSequence()
    {
        mSound.beepSequence();
    }

    public void playBeepSequenceUp()
    {
        mSound.beepSequenceUp();
    }

    public void playBuzz()
    {
        mSound.buzz();
    }

    public void playTone(int frequency, int period)
    {
        mSound.tone(frequency, period);
    }

    public void playTone(int frequency, int period, int volume)
    {
        mSound.tone(frequency, period, volume);
    }

    public void playNote(ESound sound, int frequency, int period)
    {
        switch (sound)
        {
            case PIANO:
                mSound.note(org.jabotics.robot.defs.ESound.PIANO, frequency, period);
                break;
            case FLUTE:
                mSound.note(org.jabotics.robot.defs.ESound.FLUTE, frequency, period);
                break;
            default:
                mSound.note(org.jabotics.robot.defs.ESound.XYLOPHONE, frequency, period);
                break;
        }
    }

    public void pause(int period)
    {
        mSound.pause(period);
    }
}
