/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.nxt;

import org.jabotics.robot.defs.*;

/**
 * Base class for the implementations of the sensor adapter interface running on the NXT robot.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class ASensor implements ISensorAdapter
{
    // class variables:
    private static final ASensor sInvalidAdapter = new ASensor();

    static ISensorAdapter getSensorAdapter(ESensorPort port, ESensorType type)
    {
        switch (type)
        {
            case TOUCH_SENSOR:
            return new ATouchSensor(port);

            case SOUND_SENSOR:
            return new ASoundSensor(port);

            case LIGHT_SENSOR:
            return new ALightSensor(port);

            case ULTRASONIC_SENSOR:
            return new AUltrasonicSensor(port);
            
            default:
            return sInvalidAdapter;
        }
    }

    static ISensorAdapter getButtonAdapter(EButtonType type)
    {
        return new AButton(type);
    }

    static ISensorAdapter getInvalidAdapter()
    {
        return sInvalidAdapter;
    }

    // instance variables:
    protected int mPortID = -1;
    protected ESensorType mType = ESensorType.INVALID_SENSOR;
    protected int mUpdatePeriodProlongationFactor = 1;


    protected ASensor() {} // generates invalid sensor adapter

    final public int getPortID()
    {
        return mPortID;
    }

    final public ESensorType getType()
    {
        return mType;
    }

    final public int getUpdatePeriodProlongationFactor()
    {
        return mUpdatePeriodProlongationFactor;
    }

    public float getValue()
    {
        return -1f;
    }

    public boolean hasLight()
    {
        return false;
    }

    public void lightOn()
    {
    }

    public void lightOff()
    {
    }
}
