/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.nxt;

import org.jabotics.robot.defs.*;
import lejos.nxt.SensorPort;
import lejos.nxt.UltrasonicSensor;

/**
 * AUltrasonicSensor is the implementation class for the sensor adapter interface
 * representing ultrasonic sensors of the NXT robot.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class AUltrasonicSensor extends ASensor
{
    // instance variables
    private UltrasonicSensor mSensor;

    /**
     * constructor for objects of class AUltrasonicSensor
     */
    public AUltrasonicSensor(ESensorPort port)
    {
        // initialize instance variables
        mPortID = port.ordinal();
        mType = ESensorType.ULTRASONIC_SENSOR;
        mUpdatePeriodProlongationFactor = 10;
        mSensor = new UltrasonicSensor(SensorPort.getInstance(mPortID));
        mSensor.continuous();
        /* Folgendes Zeitverhalten hat sich bei Untersuchungen gezeigt.
         * PING-Methode:
         * Nach einem Aufruf von ping() muss man mindestens 50ms warten, ehe man getDistance() aufruft.
         * Ruft man getDistance() zu früh auf, kommt es in der Methode zu einer entsprechenden Verzögerung.
         * Die reine Ausführungszeit von ping() und getDistance() beträgt zusammen 13ms. Damit beträgt
         * die kürzestmögliche Intervallzeit für diese Methode etwa 63ms. Ein Vorteil der ping-Methode
         * liegt darin, dass man mit getDistances(...) bis zu 8 Echos des Ping zurück erhält.
         * CONTINUOUS-Methode:
         * Nachdem der Sensor in den Continuous-Modus versetzt wurde, beträgt das schnellstmögliche
         * Aufrufintervall von GetDistance() etwa 35ms. Davon scheinen 30ms Wartezeit zu sein und 5ms für
         * den Aufruf selbst drauf zu gehen. Erhöht man die Wartezeit zwischen jeweils zwei Aufrufen,
         * so beträgt die Intervallzeit also Wartezeit + 5ms. Es sieht fast so aus, als würde der Aufruf
         * von getDistance automatisch die nächste Messung triggern. 
         */
    }
    
    @Override
    public float getValue()
    {
        return mSensor.getDistance();
    }
}
