/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;
import javax.swing.JFileChooser;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 * Beschreiben Sie hier die Klasse XConsole.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class XConsole implements IConsoleAdapter
{
    static final private XConsole sConsole = new XConsole();
    
    static XConsole getConsole()
    {
        return sConsole;
    }

    private final StringBuilder sb = new StringBuilder();
    private FileWriter fw = null;

    private XConsole()
    {
    }

    @Override
    public synchronized void open()
    {
        if (null != fw) return;

        JFileChooser chooser = new JFileChooser();
        int result = chooser.showSaveDialog(null);
        if (JFileChooser.APPROVE_OPTION == result)
        {
            try
            {
                File f = chooser.getSelectedFile();
                fw = new FileWriter(f);
            }
            catch (IOException e)
            {
                fw = null;
            }
        }
    }

    @Override
    public synchronized void close()
    {
        if (null == fw) return;

        try
        {
            fw.flush();
            fw.close();
            fw = null;
        }
        catch (IOException e)
        {
            fw = null;
        }
    }
    
    @Override
    public synchronized void print(String text)
    {
        if (null == fw) return;

        try
        {
            fw.write(text);
        }
        catch (IOException e)
        {
        }
    }

    @Override
    public synchronized void println(String text)
    {
        if (null == fw) return;

        try
        {
            fw.write(text);
            fw.write('\n');
        }
        catch (IOException e)
        {
        }
    }
}
