/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import java.awt.image.IndexColorModel;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
//import java.awt.image.ConvolveOp;
//import java.awt.image.Kernel;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.BasicStroke;
import java.awt.TexturePaint;
import java.awt.GradientPaint;
import java.awt.RenderingHints;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.awt.geom.AffineTransform;


/**
 * Beschreiben Sie hier die Klasse XFloor.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class XFloor extends BModelElement
{
    // coordinates of walls raster:
    private static final float X1 = -0.1f;
    private static final float X2 = 110f;
    private static final float X3 = 210f;
    private static final float X4 = 330f;
    private static final float X5 = 450f;
    private static final float X6 = 800.1f;
    private static final float Y1 = -0.1f;
    private static final float Y2 = 250f;
    private static final float Y3 = 370f;
    private static final float Y4 = 490f;
    private static final float Y5 = 600.1f;
    private static final float X3X4 = 0.5f * (X3 + X4);
    private static final float X4X5 = 0.5f * (X4 + X5);
    private static final float Y1Y2 = 0.5f * (Y1 + Y2);
    private static final float Y2Y5 = 0.5f * (Y2 + Y5);
    private static final float Y3Y4 = 0.5f * (Y3 + Y4);
    private static final float Y4Y5 = 0.5f * (Y4 + Y5);
    // luminosities (max range from 0 to 1):
    private static final float LUM1 = 1.0f;
    private static final float LUM1a = 0.95f;
    private static final float LUM2 = 0.85f;
    private static final float LUM2a = 0.8f;
    private static final float LUM2b = 0.75f;
    private static final float LUM2c = 0.7f;
    private static final float LUM2d = 0.6f;
    private static final float LUM2e = 0.55f;
    private static final float LUM2f = 0.5f;
    private static final float LUM3 = 0.45f;
    private static final float LUM4 = 0.3f;
    private static final float LUM4a = 0.38f;
    private static final float LUM5 = 0.2f;
    private static final float LUM6 = 0.0f;
    // luminosity of darkest place in scene and resulting luminosity range:
    private static final float LUM_MIN = 0.2f;
    private static final float LUM_RANGE = 1f - LUM_MIN;
    
    // rectangles for painting:
    private static final Rectangle.Float BOUNDS = new Rectangle.Float(X1,Y1,X6-X1,Y5-Y1); // small overlap with walls
    private static final Rectangle.Float R1 = new Rectangle.Float(X1,Y1,X5-X1,Y2-Y1);
    private static final Rectangle.Float R2 = new Rectangle.Float(X5,Y1,X6-X5,Y5-Y1);
    private static final Rectangle.Float R3 = new Rectangle.Float(X3,Y3,X5-X3,Y5-Y3);
    private static final Rectangle.Float R4 = new Rectangle.Float(X2,Y3,X3-X2,Y4-Y3);
    private static final Rectangle.Float R5 = new Rectangle.Float(X1,Y2,X2-X1,Y5-Y2);
    private static final Rectangle.Float R6 = new Rectangle.Float(X2,Y4,X3-X2,Y5-Y4);
    private static final Rectangle.Float R7 = new Rectangle.Float(X2,Y2,X5-X2,Y3-Y2);

    private AffineTransform mAfT;
    private BufferedImage   mSceneImgMaxBrightness;
    private WritableRaster  mBrightnessRaster;

    public XFloor()
    {
        super();
        buildGraphics();
    }

    private void buildGraphics()
    {
        // temporary image:
        BufferedImage img = new BufferedImage(2,2,BufferedImage.TYPE_INT_RGB);
        int rgbWarmesGelb = 255*256*256+230*256+160;
        //int rgbKlassischesBlau = 207*256*256+231*256+245;
        int rgbOrange10 = 255*256*256+204*256+153;
        img.setRGB(0,0,rgbWarmesGelb);
        img.setRGB(1,1,rgbWarmesGelb);
        //img.setRGB(0,1,rgbKlassischesBlau);
        //img.setRGB(1,0,rgbKlassischesBlau);
        img.setRGB(0,1,rgbOrange10);
        img.setRGB(1,0,rgbOrange10);

        XGraphicsShape shape_floor = new XGraphicsShape(BOUNDS);
        TexturePaint texture = new TexturePaint(img,new Rectangle2D.Float(0f,0f,100f,100f));
        shape_floor.setPaint(texture);
        shape_floor.setDraw(false);
        shape_floor.setFill(true);
        mGraphicsRep.add(shape_floor);

        //Color dark_blue = new Color(32,64,128);
        Color mid_gray = new Color(96,96,96);
        BasicStroke thin_stroke = new BasicStroke(3,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);
        BasicStroke thick_stroke = new BasicStroke(6,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);

        XGraphicsShape shape_rect = new XGraphicsShape(new Rectangle.Float(550f,270f,160f,240f));
        shape_rect.setStroke(thick_stroke);
        shape_rect.setPaint(mid_gray);
        shape_rect.setDraw(true);
        shape_rect.setFill(false);
        mGraphicsRep.add(shape_rect);

        Path2D.Float path_line = new Path2D.Float(Path2D.WIND_EVEN_ODD,5);
        path_line.moveTo(310f,180f);
        path_line.curveTo(250f,140f,200f,60f,130f,60f);
        path_line.curveTo(60f,60f,60f,140f,80f,170f);
        path_line.curveTo(100f,200f,150f,160f,210f,110f);
        path_line.curveTo(270f,60f,300f,70f,330f,90f);
        XGraphicsShape shape_line = new XGraphicsShape(path_line);
        shape_line.setPaint(mid_gray);
        shape_line.setStroke(thin_stroke);
        shape_line.setDraw(true);
        shape_line.setFill(false);
        mGraphicsRep.add(shape_line);

        Path2D.Float path_line2 = new Path2D.Float(Path2D.WIND_EVEN_ODD,8);
        path_line2.moveTo(410f,110f);
        path_line2.curveTo(390f,150f,440f,180f,480f,180f);
        path_line2.curveTo(520f,180f,510f,120f,540f,120f);
        path_line2.curveTo(570f,120f,600f,190f,660f,190f);
        path_line2.curveTo(720f,190f,740f,140f,710f,100f);
        path_line2.curveTo(680f,60f,640f,60f,560f,60f);
        path_line2.curveTo(480f,60f,430f,70f,410f,110f);
        path_line2.closePath();
        XGraphicsShape shape_line2 = new XGraphicsShape(path_line2);
        shape_line2.setPaint(mid_gray);
        shape_line2.setStroke(thin_stroke);
        shape_line2.setDraw(true);
        shape_line2.setFill(false);
        mGraphicsRep.add(shape_line2);

        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // prepare image for measuring brightness
        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        /***********************************************************************************
        // Old approach: Calculate high resolution (1mm/pixel) brightness image that is
        // softended afterwards. The brightness of the floor will be directly read from
        // image without any further data manipulation

        float d00 = 1.000f/3.14f; // Gauß (normalized, gives reasonable width for resolution of 1mm/px): norm*exp(-(x-x0)²-(y-y0)²)
        float d01 = 0.368f/3.14f;
        float d11 = 0.135f/3.14f;
        float d02 = 0.018f/3.14f;
        float d12 = 0.007f/3.14f;
        float[] convolutionMatrix = {   0,d12,d02,d12,0,
                                      d12,d11,d01,d11,d12,
                                      d02,d01,d00,d01,d02,
                                      d12,d11,d01,d11,d12,
                                        0,d12,d02,d12,0   };
        int scale = 10;
        int width = scale * 804;
        int height = scale * 604;
        // paint into temporary image:
        img = new BufferedImage(width,height,BufferedImage.TYPE_BYTE_GRAY);
        img.setAccelerationPriority(1); // highest priority for acceleration while rendering
        Graphics2D g2D = img.createGraphics();
        mAfT = new AffineTransform((float)scale, 0f, 0f, (float)scale, (float)(scale * 1), (float)(scale * 1)); // border of 1cm
        g2D.setTransform(mAfT);
        this.paint(g2D);

        // perform blur operation to model limitted spatial resolution of light sensor
        mSceneImgMaxBrightness = new BufferedImage(width,height,BufferedImage.TYPE_BYTE_GRAY);
        mSceneImgMaxBrightness.setAccelerationPriority(1); // highest priority for acceleration while rendering
        ConvolveOp blurOperation = new ConvolveOp(new Kernel(5,5,convolutionMatrix));
        blurOperation.filter(img.getRaster(),mSceneImgMaxBrightness.getRaster());
        img.setAccelerationPriority(0); // do not wait for garbage collection to kick out of video memory
        mSceneImgMaxBrightness.setAccelerationPriority(0); // acceleration no longer required => kick out of video memory
        ***********************************************************************************/

        
        /***********************************************************************************
         * New approach: calculated brightness image with slightly reduced resolution
         * (2mm/pixel) The brightness of the floor will be determined from the image
         * by calculating a weighted average of a close vicinity of the sensor location
         * (3x3 pixels corresponding to a sensitive diameter auf about 4mm, see
         * getMaxBrightness(float x, float y) for details). A continuous change of
         * detected sensor values is possible with this approach.
         * Further advantages: 
         * - No time consuming softening at programm startup. The softening of every single
         *   pixel (about 50 MPixels!) of the 1mm-resolution brightness image basically
         *   corresponds to 50,000,000 just-in-time calculations of the floor brightness
         *   for the new approach. With a frequency limited to 200 measurements per second,
         *   this corresponds to a lifetime of 50,000,000/200 = 250,000 seconds.
         * - Reduced memory consumption due to reduced resolution of brightness image
         * - No temporary image (and thus memory) required for softening procedure.
         ***********************************************************************************/
        int scale = 5;
        int width = scale * 802;
        int height = scale * 602;
        // paint into temporary image:

        mSceneImgMaxBrightness = new BufferedImage(width,height,BufferedImage.TYPE_BYTE_GRAY);
        mSceneImgMaxBrightness.setAccelerationPriority(1); // highest priority for acceleration while rendering
        Graphics2D g2D = mSceneImgMaxBrightness.createGraphics();
        g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        //mAfT = new AffineTransform((float)scale, 0f, 0f, (float)scale, (float)(scale * 1), (float)(scale * 1)); // border of 1cm
        mAfT = new AffineTransform(scale, 0, 0, scale, (scale * 1), (scale * 1)); // border of 1cm
        g2D.setTransform(mAfT);
        this.paint(g2D);
        mSceneImgMaxBrightness.setAccelerationPriority(0); // acceleration no longer required => kick out of video memory
        mBrightnessRaster = mSceneImgMaxBrightness.getRaster();
    }

    @Override
    public void manageVisitor(BModelVisitor visitor)
    {
        visitor.visitFloor(this);
    }

    void paintLuminosity(Graphics2D g, float min_luminosity)
    {
        float lum_range = 1f - min_luminosity;
        Color c1 = new Color(0f, 0f, 0f, 0f);
        Color c2 = new Color(0f, 0f, 0f, 1f - min_luminosity - lum_range * LUM2);
        Color c3 = new Color(0f, 0f, 0f, 1f - min_luminosity - lum_range * LUM3);
        Color c4 = new Color(0f, 0f, 0f, 1f - min_luminosity - lum_range * LUM4);
        Color c5 = new Color(0f, 0f, 0f, 1f - min_luminosity - lum_range * LUM5);
        Color c6 = new Color(0f, 0f, 0f, 1f - min_luminosity - lum_range * LUM6);

        Paint paint = g.getPaint();

        g.setPaint(new GradientPaint(X1, 0f, c1, X5, 0f, c2));
        g.fill(R1);
        g.setPaint(c2);
        g.fill(R2);
        g.setPaint(new GradientPaint(X5, 0f, c2, X3, 0f, c3));
        g.fill(R3);
        g.setPaint(new GradientPaint(X3, 0f, c3, X2, 0f, c4));
        g.fill(R4);
        g.setPaint(c4);
        g.fill(R5);
        g.setPaint(new GradientPaint(X2, 0f, c4, X3, 0f, c5));
        g.fill(R6);
        g.setPaint(new GradientPaint(X2, 0f, c4, X5, 0f, c6));
        g.fill(R7);

        g.setPaint(paint);
    }

    private Color createColor(int r_max, int g_max, int b_max, int a_max, float min_fraction, float depth_fraction, float rel_fraction)
    {
        float fraction = min_fraction + depth_fraction * rel_fraction;
        int r = (int) (fraction * r_max);
        int g = (int) (fraction * g_max);
        int b = (int) (fraction * b_max);
        int a = (int) (fraction * a_max);
        return new Color(r,g,b,a);
    }

    private float calcLuminosityLin(float pos, float lum1, float pos1, float lum2, float pos2)
    {
        // caluculates luminosity for position pos between pos1 with lum1 and pos2 with lum2
        return lum1 + (lum2 - lum1) * (pos - pos1) / (pos2 - pos1);
    }

    private float calcLuminosityQuad(float pos, float lum_max, float pos_max, float lum_min, float pos_min)
    {
        float delta_lum = lum_max - lum_min;
        float delta_max = pos_max - pos_min;
        float u = (pos_max - pos) / delta_max;
        return lum_min + delta_lum * (1f - u * u);
    }

    float getLuminosityMaxRange(float x, float y)
    {
        if (x > X5) return LUM2;
        else if (y < Y2) return this.calcLuminosityLin(x,LUM1,X1,LUM2,X5);
        else if (x < X2) return LUM4;
        else if (y < Y3) return this.calcLuminosityLin(x,LUM4,X2,LUM6,X5);
        else if (x > X3) return this.calcLuminosityLin(x,LUM2,X5,LUM3,X3);
        else if (y < Y4) return this.calcLuminosityLin(x,LUM3,X3,LUM4,X2);
        else return this.calcLuminosityLin(x,LUM4,X2,LUM5,X3);
    }

    float getEdgeLuminosityMaxRange(float x, float y)
    {
        if (y < 0.1f) return this.calcLuminosityLin(x,LUM1a,X1,LUM2a,X6);
        else if (X6 - x < 0.2f) return this.calcLuminosityQuad(y,LUM2,Y1,LUM2a,Y5);
        else if (y < Y2)
        {
            if (x < 0.1f) return this.calcLuminosityQuad(y,LUM1,Y1Y2,LUM1a,Y1);
            else  return this.calcLuminosityLin(x,LUM1a,X1,LUM2,X5);
        }
        else if (x > X5) return LUM2a;
        else if (x < X2)
        {
            if (x < 0.1f) return this.calcLuminosityQuad(y,LUM4a,Y2Y5,LUM4,Y2);
            else return LUM4;
        }
        else if (y < Y3) return this.calcLuminosityLin(x,LUM4,X2,LUM6,X5);
        else if ((x < X3) && (y > Y4)) return this.calcLuminosityLin(x,LUM4,X2,LUM5,X3);
        else if ((x < X3X4) && (y > Y3Y4) && (x - X3 < Y5 - y))
        {
            if (X3 - x > y - Y4) return this.calcLuminosityLin(x,LUM3,X3,LUM4,X2);
            else return this.calcLuminosityLin(y,LUM2d,Y5,LUM2e,Y4);
        }
        else if ((x < X4) && (y < Y4Y5))
        {
            if (X4 - x > y - Y3) return this.calcLuminosityLin(x,LUM2f,X4,LUM4,X2);
            else return this.calcLuminosityLin(y,LUM2e,Y4,LUM2f,Y3);
        }
        else if ((y > Y4Y5) && (X5 - x > Y5 - y)) return this.calcLuminosityLin(x,LUM2c,X5,LUM2d,X3);
        else if ((x > X4X5) && (y > Y3Y4)) return this.calcLuminosityLin(y,LUM2b,Y4,LUM2c,Y5);
        else if (x - X4 > y - Y3) return this.calcLuminosityLin(x,LUM2a,X5,LUM2b,X4);
        else return LUM2b;
    }

    float getLuminosity(float x, float y)
    {
        return LUM_MIN + LUM_RANGE * getLuminosityMaxRange(x,y);
    }

    float getEdgeLuminosity(float x, float y)
    {
        return LUM_MIN + LUM_RANGE * getEdgeLuminosityMaxRange(x,y);
    }

    /*float getBrightness(float x, float y)
    {
        return this.getLuminosity(x,y) * this.getMaxBrightness(x,y);
    }*/

    float getMaxBrightness(float x, float y)
    {
        /***********************************************************************************
        // Old approach (see above);
        Point2D.Float ptModel = new Point2D.Float(x,y);
        Point2D ptImage = mAfT.transform(ptModel, null);
        try
        {
            // grayscale image => only one band (index 0) ;    should result in values between 0 and 100
            return ((float)mSceneImgMaxBrightness.getRaster().getSample((int)ptImage.getX(),(int)ptImage.getY(),0)) / 2.55f;
        }
        catch (ArrayIndexOutOfBoundsException e)
        {
            return 0f;
        }
         ***********************************************************************************/

         
        /***********************************************************************************
         * New approach (see above);
         ***********************************************************************************/
         float[] coordinates = {x,y};
         mAfT.transform(coordinates, 0, coordinates, 0, 1);
         int nx = (int)coordinates[0]; // index of center ...
         int ny = (int)coordinates[1]; //    ... in 3x3 raster
         float dx = (float)nx + 0.5f - coordinates[0]; // shift of center of 3x3 raster ...
         float dy = (float)ny + 0.5f - coordinates[1]; //    ... relative to reference point

         float[] dxSqr = { (dx - 1f) * (dx - 1f),
                            dx * dx,
                           (dx + 1f) * (dx + 1f) };

         float[] dySqr = { (dy - 1f) * (dy - 1f),
                            dy * dy,
                           (dy + 1f) * (dy + 1f) };

         float[][] weight = new float[3][3];

         float normalization = 0f;
         for (int i = 0; i < 3; ++i)
         {
             for (int j = 0; j < 3; ++j)
             {
                 float drSqr = dxSqr[i] + dySqr[j];
                 float w = 1f - 2f * drSqr / (2.25f + drSqr);
                 w = (w < 0f) ? 0f : w*w;
                 normalization += w;
                 weight[i][j] = w;
             }
         }

        float brightness = 0f;
        nx -= 1; // shift index to upper left
        ny -= 1; // corner of the 3x3 raster
        try
        {
             for (int i = 0; i < 3; ++i)
             {
                 for (int j = 0; j < 3; ++j)
                 {
                     brightness += weight[i][j] * mBrightnessRaster.getSampleFloat(nx + i, ny + j, 0);
                 }
             }
        }
        catch (ArrayIndexOutOfBoundsException e)
        {
            return 0f;
        }

         // should result in values between 0 and 100:
         return brightness / (2.55f * normalization);
    }
}