/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.swing.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Beschreiben Sie hier die Klasse XInfoPanel.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class XInfoPanel extends JPanel implements XLogStreamTextArea.ILogListener
{
    class SimulationStateChangeHandler implements XSimulation.IListener
    {
        @Override
        public void stateChanged(XSimulation.EState oldState, XSimulation.EState newState)
        {
            if (XSimulation.EState.TERMINATING == newState)
            {
                mOutErrPanel.stopLogging();
            }
        }
    }

    // instance variables
    private boolean mInitialized = false;

    private JTabbedPane mTappedPane;
    private XOutErrStreamPanel mOutErrPanel;

    public XInfoPanel(JTabbedPane tappedPane)
    {
        super(new GridBagLayout());

        mTappedPane = tappedPane;

        mOutErrPanel = new XOutErrStreamPanel();
        Dimension dimMin = new Dimension(210,346);
        Dimension dimMax = new Dimension(600,346);
        mOutErrPanel.setPreferredSize(dimMin);
        mOutErrPanel.setMinimumSize(dimMin);
        mOutErrPanel.setMaximumSize(dimMax);

        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.insets = new Insets(1,1,1,1);
        this.add(mOutErrPanel,c);

        mOutErrPanel.getErrTA().addListener(this);
    }

    public void init()
    {
        if (mInitialized) return;
        mInitialized = true;

        mOutErrPanel.getSplitPane().setDividerLocation(1.0);

        PrintStream originalOut = System.out;
        PrintStream originalErr = System.err;

        try
        {
            PipedOutputStream osOut = new PipedOutputStream();
            PipedInputStream isOut = new PipedInputStream(osOut);
            PrintStream psOut = new PrintStream(osOut);
            System.setOut(psOut);

            PipedOutputStream osErr = new PipedOutputStream();
            PipedInputStream isErr = new PipedInputStream(osErr);
            PrintStream psErr = new PrintStream(osErr);
            System.setErr(psErr);

            mOutErrPanel.startLogging(isOut, isErr);
    
            XSimulation.addListener(new SimulationStateChangeHandler());
        }
        catch (IOException ioe)
        {
            System.setOut(originalOut);
            System.setErr(originalErr);
        }

        System.out.print("www.jabotics.org (©");
        System.out.print(XMsg.COPYRIGHT);
        System.out.println(")");
        System.out.print(XMsg.VERSION);
        System.out.print(": ");
        System.out.println(XMsg.VERSION_NUMBER);
        System.out.print(XMsg.AUTHOR);
        System.out.println(": Frank Gollnik");
        System.out.print(XMsg.PUBLISHED_UNDER);
        System.out.println(": GNU GPLv3\n");
    }

    public void logEvent(XLogStreamTextArea source)
    {
        if (!mTappedPane.getSelectedComponent().equals(this))
        {
            mTappedPane.setSelectedComponent(this);
        }
    }
}
