/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.D;
import org.jabotics.robot.defs.ISessionData;
import org.jabotics.robot.compile.*;
import org.jabotics.robot.util.XTimedThread;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.border.LineBorder;
//import java.util.concurrent.atomic.AtomicBoolean;


/**
 * Beschreiben Sie hier die Klasse XSceneView.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class XSceneView extends XSpriteView
{
    // constants:
    public final static int MIN_SCALING = 1;
    public final static int MAX_SCALING = 10;
    private final static int PREFERRED_LENGTH = 500; // px
    // parameters for auto-scroll:
    private final static float DIST1 = 120f;   // no scrolling within 120 px from center
    private final static float DIST2 = 200f;   // scrolling speed increasing from 0 at DIST1 to V_SCROLL_MAX at DIST2
    private final static float DIST2REL = DIST2 / DIST1;
    private final static float V_SCROLL_FACTOR = 1f / (DIST2REL * DIST2REL - 1f);
    private final static float V_SCROLL_MAX = 0.01f;


    private class DragInfo
    {
        boolean mRunning = false;
        boolean mRotate = false;
        int mShiftX = 0;
        int mShiftY = 0;
        float mStartPhi = 0f;
        float mStartPhiRobot = 0f;
    }

    private class MouseHandler extends MouseAdapter
    {
        public void mousePressed(MouseEvent e)
        {
            mPtCursor = new Point(e.getX(),e.getY());

            if ( ( ( XSimulation.getState() != XSimulation.EState.RUNNING)
                  || XPreferences.getReadInstance().getSimBreaksEnabled() )
               && (e.getButton() == MouseEvent.BUTTON1) )
            {
                XRobot.XDynamicData robotDD = mRobot.getDynamicData();

                Point ptRobot = transformPhysicalPosToScreen(robotDD.getRobotX(),robotDD.getRobotY());
                int dx = ptRobot.x - mPtCursor.x;
                int dy = ptRobot.y - mPtCursor.y;
                int dSqr = dx*dx + dy*dy;
                if (dSqr > 14400) return;

                DragInfo info = new DragInfo();
                info.mRotate = (dSqr > 2500);

                if (info.mRotate)
                {
                    info.mStartPhi = getCurrentPhi(mPtCursor,robotDD);
                    info.mStartPhiRobot = robotDD.getRobotPhi();
                    getRootPane().setCursor(mCursorRotate);
                }
                else
                {
                    info.mShiftX = dx;
                    info.mShiftY = dy;
                    getRootPane().setCursor(mCursorMove);
                }

                info.mRunning = XSimulation.pauseSimulation();
                mRobot.enterDragMode();
                mDragInfo = info;
            }
        }

        public void mouseReleased(MouseEvent e)
        {
            if (null != mDragInfo)
            {
                boolean running = mDragInfo.mRunning;
                mDragInfo = null;
                mRobot.leaveDragMode();
                if (running) XSimulation.resumeSimulation();
            }

            if (getBounds().contains(e.getX(), e.getY()))
            {
                mouseMoved(e);
            }
            else
            {
                getRootPane().setCursor(Cursor.getDefaultCursor());
                mPtCursor = null;
            }
        }

        public void mouseMoved(MouseEvent e)
        {
            mPtCursor = new Point(e.getX(),e.getY());
        }

        public void mouseDragged(MouseEvent e)
        {
            mPtCursor = new Point(e.getX(),e.getY());
        }

        public void mouseEntered(MouseEvent e)
        {
            mPtCursor = new Point(e.getX(),e.getY());
        }

        public void mouseExited(MouseEvent e)
        {
            if (null == mDragInfo)
            {
                mPtCursor = null;
                if (getCursor().getType() != Cursor.DEFAULT_CURSOR)
                {
                    getRootPane().setCursor(Cursor.getDefaultCursor());
                }
            }
        }
    }

    private class TrackGraphicsHandler implements Runnable, ISessionData
    {
        long mNextUpdateTime = 0;

        private TrackGraphicsHandler()
        {
            XSession.getSession().add(this);
        }

        public void run()
        {
            synchronized(mViewCenter)
            {
                Point ptCursor = mPtCursor;
                DragInfo info = mDragInfo;
                XRobot.XDynamicData robotDD = mRobot.getDynamicData();
                
                if (null != ptCursor)
                {
                    if (null == info)
                    {
                        // no dragging
        
                        if ( ( XSimulation.getState() != XSimulation.EState.RUNNING)
                            || XPreferences.getReadInstance().getSimBreaksEnabled() )
                        {
                            Cursor cursor = getCursor();
                            int cursorType = cursor.getType();
                            Point ptRobot = transformPhysicalPosToScreen(robotDD.getRobotX(),robotDD.getRobotY());
                            int dx = ptRobot.x - ptCursor.x;
                            int dy = ptRobot.y - ptCursor.y;
                            int dSqr = dx*dx + dy*dy;
                            if (dSqr > 14400)
                            {
                                if (cursorType != Cursor.DEFAULT_CURSOR)
                                {
                                    getRootPane().setCursor(Cursor.getDefaultCursor());
                                }
                            }
                            else if (dSqr > 2500)
                            {
                                if (cursor != mCursorCanRotate)
                                {
                                    getRootPane().setCursor(mCursorCanRotate);
                                }
                            }
                            else
                            {
                                if (cursor != mCursorCanMove)
                                {
                                    getRootPane().setCursor(mCursorCanMove);
                                }
                            }
                        }
                    }
                    else
                    {
                        // Dragging
                        boolean dragged = false;
                        if (info.mRotate)
                        {
                            float currentPhi = getCurrentPhi(ptCursor,robotDD); // values: 0 <= phi < TWO_PI_F
                            float dPhi = restrictedAngle(currentPhi - info.mStartPhi); // values: 0 <= phi < TWO_PI_F
                            float phi = restrictedAngle(info.mStartPhiRobot + dPhi); // values: 0 <= phi < TWO_PI_F
                            float phiRobot = robotDD.getRobotPhi();
                            dragged = mRobot.setDraggedOrientation(phi);
                            if (!dragged)
                            {
                                float dPhiRobot = restrictedAngle(phiRobot - info.mStartPhiRobot);
                                dPhi = restrictedAngle(dPhi - dPhiRobot); // missing part relative to target orientation
                                // try small step of about 0.01 rad
                                if (dPhi == Math.PI)
                                {
                                    dPhi = 0.01f;
                                    phi = restrictedAngle(phiRobot + dPhi);
                                    dragged = mRobot.setDraggedOrientation(phi);
                                    if (!dragged)
                                    {
                                        dPhi = -0.01f;
                                        phi = restrictedAngle(phiRobot + dPhi);
                                        dragged = mRobot.setDraggedOrientation(phi);
                                    }
                                }
                                else
                                {
                                    dPhi = (dPhi < D.PI_F) ? 0.01f : -0.01f;
                                    phi = restrictedAngle(phiRobot + dPhi);
                                    dragged = mRobot.setDraggedOrientation(phi);
                                }
                            }
                        }
                        else
                        {
                            Point.Float ptRobot = transformScreenPosToPhysical(ptCursor.x + info.mShiftX, ptCursor.y + info.mShiftY);
                            dragged = mRobot.setDraggedPosition(ptRobot.x,ptRobot.y);
                            if (!dragged)
                            {
                                // try smaller step
                                float robotX = robotDD.getRobotX();
                                float robotY = robotDD.getRobotY();
                                float totalStepX = ptRobot.x - robotX;
                                float totalStepY = ptRobot.y - robotY;
                                float norm = 0.5f / (float)Math.sqrt(totalStepX * totalStepX + totalStepY * totalStepY + 0.0001); // => step of about 0.5 mm
                                float stepX = norm * totalStepX;
                                float stepY = norm * totalStepY;
                                dragged = mRobot.setDraggedPosition(robotX + stepX, robotY + stepY);
                                if (!dragged)
                                {
                                    // try small step in x-direction
                                    dragged = mRobot.setDraggedPosition(robotX + stepX, robotY);
                                    if (!dragged)
                                    {
                                        // try small step in y-direction
                                        dragged = mRobot.setDraggedPosition(robotX, robotY + stepY);
                                    }
                                }
                            }
                        }

                        robotDD = mRobot.getDynamicData(); // update after possible translation or rotation
                        if (dragged && !info.mRunning)
                        {
                            XPreferences.Write pw = XPreferences.getWriteInstance();
                            pw.setRobotStartX(robotDD.getRobotX());
                            pw.setRobotStartY(robotDD.getRobotY());
                            pw.setRobotStartPhi(robotDD.getRobotPhi());
                            pw.store();
                        }
                    }
                }

                float hotspotX;
                float hotspotY;
                if ((null != info) && (!info.mRotate))
                {
                    Point.Float ptCursorPhysical = transformScreenPosToPhysical(ptCursor.x + info.mShiftX,ptCursor.y + info.mShiftY);
                    hotspotX = (ptCursorPhysical.x < 0) ? 0f : ((ptCursorPhysical.x > XScene.BOUNDS.width) ? XScene.BOUNDS.width : ptCursorPhysical.x);
                    hotspotY = (ptCursorPhysical.y < 0) ? 0f : ((ptCursorPhysical.y > XScene.BOUNDS.height) ? XScene.BOUNDS.height : ptCursorPhysical.y);
                }
                else
                {
                    hotspotX = robotDD.getRobotX();
                    hotspotY = robotDD.getRobotY();
                }

                float scaleOverDist1 = getScale() / DIST1;
    
                float dxRel = (mViewCenter.x - hotspotX) * scaleOverDist1;
                float vScrollXRel = (dxRel * dxRel - 1f) * V_SCROLL_FACTOR;
                float vScrollX = (vScrollXRel < 0) ? 0f : ((vScrollXRel > 1f) ? V_SCROLL_MAX : vScrollXRel * V_SCROLL_MAX);
    
                float dyRel = (mViewCenter.y - hotspotY) * scaleOverDist1;
                float vScrollYRel = (dyRel * dyRel - 1f) * V_SCROLL_FACTOR;
                float vScrollY = (vScrollYRel < 0) ? 0f : ((vScrollYRel > 1f) ? V_SCROLL_MAX : vScrollYRel * V_SCROLL_MAX);
    
                float x = (1 - vScrollX) * mViewCenter.x + vScrollX * hotspotX;
                float y = (1 - vScrollY) * mViewCenter.y + vScrollY * hotspotY;
                mViewCenter = new Point.Float(x,y);
            }

            // do not call updateGraphics() (resulting in repaint), if no painting has been performed since last update request
            if (XTimer.currentTimeMillis() >= mNextUpdateTime)
            {
                updateGraphics();
                if (CS.TIMED_THREAD_DILATATION)
                {
                    mNextUpdateTime = XTimer.currentTimeMillis() + (long)(XTimedThread.getTimeDilatation() * XSimulationView.VIEW_UPDATE_DELAY_F);
                }
                else
                {
                    mNextUpdateTime = XTimer.currentTimeMillis() + XSimulationView.VIEW_UPDATE_DELAY;
                }
            }
        }

        @Override
        public String getKey()
        {
            return "XSceneView.TrackGraphicsHandler";
        }
    
        @Override
        public void stopSession()
        {
        }
    
        @Override
        public ISessionData resetSession()
        {
            return this;
        }
    
        @Override
        public void startSession()
        {
            mNextUpdateTime = XTimer.currentTimeMillis();
        }
    }


    // instance variables:
    private final TrackGraphicsHandler mTrackGraphicsHandler = new TrackGraphicsHandler();
    private final XSimulationView mSimView;
    private final XRobot mRobot;
    private volatile XRobot.XDynamicData mRobotDD;
    private volatile Point.Float mViewCenter;
    // Used for D&D:
    private final Cursor mCursorCanMove;
    private final Cursor mCursorMove;
    private final Cursor mCursorCanRotate;
    private final Cursor mCursorRotate;
    private volatile Point mPtCursor = null;
    private volatile DragInfo mDragInfo = null;

    // Constructors:
    XSceneView(XSimulationView simulationView, boolean interactive)
    {
        super(); // initialization of XSpriteView cannot be completed yet => call to init(...) below will do that

        mSimView = simulationView;
        mRobot = XSimulationModel.getRobot();
        mRobotDD = mRobot.getDynamicData();
        XSimulationModel.updateGraphics(mRobotDD);
        mViewCenter = new Point.Float(mRobotDD.getRobotX(), mRobotDD.getRobotY());

        super.init(XScene.BOUNDS,
                   mRobot.getEnclosingRect(null),
                   calculateScale(XPreferences.getReadInstance().getViewScaling()),
                   mViewCenter,
                   mViewCenter );

        Dimension dim = new Dimension(PREFERRED_LENGTH,PREFERRED_LENGTH);
        this.setPreferredSize(dim);
        this.setMinimumSize(dim);
        this.setBorder(new LineBorder(Color.black,1));
        this.setBackground(Color.lightGray);

        if (interactive)
        {
            MouseHandler mouseHandler = new MouseHandler();
            this.addMouseListener(mouseHandler);
            this.addMouseMotionListener(mouseHandler);
        }

        boolean isWindows = ('\\' == java.io.File.separatorChar);

        ClassLoader cl = this.getClass().getClassLoader();
        Image imgCanMove =   isWindows ? (new ImageIcon(cl.getResource("resources/images/CanMove32x32.gif"))).getImage()
                                       : (new ImageIcon(cl.getResource("resources/images/CanMove24x24.gif"))).getImage();
        Image imgMove =      isWindows ? (new ImageIcon(cl.getResource("resources/images/Move32x32.gif"))).getImage()
                                       : (new ImageIcon(cl.getResource("resources/images/Move24x24.gif"))).getImage();
        Image imgCanRotate = isWindows ? (new ImageIcon(cl.getResource("resources/images/CanRotate32x32.gif"))).getImage()
                                       : (new ImageIcon(cl.getResource("resources/images/CanRotate24x24.gif"))).getImage();
        Image imgRotate =    isWindows ? (new ImageIcon(cl.getResource("resources/images/Rotate32x32.gif"))).getImage()
                                       : (new ImageIcon(cl.getResource("resources/images/Rotate24x24.gif"))).getImage();
        Point hotSpot = new Point(12,12);
        Toolkit tk = Toolkit.getDefaultToolkit();
        mCursorCanMove = tk.createCustomCursor(imgCanMove,hotSpot,"RobotCanMove");
        mCursorMove = tk.createCustomCursor(imgMove,hotSpot,"RobotMove");
        mCursorCanRotate = tk.createCustomCursor(imgCanRotate,hotSpot,"RobotCanRotate");
        mCursorRotate = tk.createCustomCursor(imgRotate,hotSpot,"RobotRotate");
    }

    void trackGraphics()
    {
        SwingUtilities.invokeLater(mTrackGraphicsHandler);
    }

    private void updateGraphics()
    {
        mRobotDD = mRobot.getDynamicData();
        XSimulationModel.updateGraphics(mRobotDD);
        setSpritePosition(new Point.Float(mRobotDD.getRobotX(), mRobotDD.getRobotY()),true); // forceRepaintSprite = true
        setViewCenter(mViewCenter);
    }

    void setScaling(int scaling)
    {
        if ( (scaling <  MIN_SCALING) || (scaling > MAX_SCALING)) return;
        setScale(calculateScale(scaling));
    }

    static private float calculateScale(int scaling)
    {
        return 0.6f * (float)Math.exp( 0.2 * ( (scaling <  MIN_SCALING) ? MIN_SCALING : ( (scaling > MAX_SCALING) ? MAX_SCALING : scaling) ) );
    }

    /*
     * returns values in the range 0 <= phi < TWO_PI as long as angle is not too large / too small
     */
    private float restrictedAngle(float angle)
    {
        //return ((angle >= D.TWO_PI) || (angle < 0)) ? (angle - D.TWO_PI * Math.floor(angle/D.TWO_PI)) : angle;
        if (angle < 0) return angle + D.TWO_PI_F;
        if (angle >= D.TWO_PI_F) return angle - D.TWO_PI_F;
        return angle;
    }

    /*
     * returns values in the range 0 <= phi < TWO_PI
     */
    private float getCurrentPhi(Point ptCursor, XRobot.XDynamicData dd)
    {
        Point ptRobot = transformPhysicalPosToScreen(dd.getRobotX(), dd.getRobotY());
        int dx = ptRobot.x - ptCursor.x;
        int dy = ptRobot.y - ptCursor.y;
        if ( (0 == dx) && (0 == dy) ) return 0f;
        if (0 == dx)
        {
            return (dy < 0) ? 1.5f*D.PI_F : 0.5f*D.PI_F;
        }
        else
        {
            double tan = ((double)dy) / ((double)dx);
            float atan = (float)Math.atan(tan);
            return (dx < 0) ? D.PI_F + atan : ( (tan < 0) ? D.TWO_PI_F + atan : atan );
        }
    }

    @Override
    protected void paintBackgroundImage(Graphics2D g2D)
    {
        XScene scene = XSimulationModel.getScene();
        XFloor floor = scene.getFloor();
        g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        floor.paint(g2D); // paints with opaque colors (=> alpha = 255)
        g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);
        floor.paintLuminosity(g2D,0.7f);
        g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        scene.getWall().paint(g2D);
    }

    @Override
    protected void paintSprite(Graphics2D g2D)
    {
        g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        mRobot.paint(g2D);
    }
}
