/********************************************************************************************
 *
 * Copyright 2012-2016  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.geom.Rectangle2D;

/**
 * Beschreiben Sie hier die Klasse XTouchSensor.
 * 
 * @author  Frank Gollnik
 * @version 1.0.3
 */
public class XTouchSensor extends BSensor
{
    // Klassenvariablen:
    private static final XTouchSensor sTouchSensorFront = new XTouchSensor(ESensorDirection.FRONT);
    private static final XTouchSensor sTouchSensorBack = new XTouchSensor(ESensorDirection.BACK);
    private static final Color PAINT_SENSOR_IS_PRESSED = new Color(192,0,0);

    static public XTouchSensor getTouchSensor(ESensorDirection direction)
    {
        switch (direction)
        {
            case FRONT: return sTouchSensorFront;
            default:    return sTouchSensorBack;
        }
    }

    // Instanzvariablen:
    private volatile int mValue = 0;
    private XGraphicsShape mGraphicsSensor;

    private XTouchSensor(ESensorDirection direction)
    {
        super();
        mDirection = direction;
        buildGraphics();
    }

    private void buildGraphics()
    {
        mGraphicsSensor = new XGraphicsShape(new Rectangle2D.Float(-0.3f,-4.2f,0.6f,8.4f));
        mGraphicsSensor.setPaint(Color.darkGray);
        mGraphicsSensor.setDraw(false);
        mGraphicsSensor.setFill(true);
        mGraphicsRep.add(mGraphicsSensor);
    }

    public void updateGraphics()
    {
        mGraphicsSensor.setFill(this.isReferenced());
        if (0 == mValue)
        {
            mGraphicsSensor.setPaint(Color.darkGray);
        }
        else
        {
            mGraphicsSensor.setPaint(Color.red);
        }
    }

    @Override
    public ESensorType getType()
    {
        return ESensorType.TOUCH_SENSOR;
    }

    @Override
    public float getValue(XSensorRef ref)
    {
        return mValue;
    }

    protected void setValue(int value)
    {
        if ((0 == value) || (1 == value))  mValue = value;
    }

    @Override
    public void manageVisitor(BModelVisitor visitor)
    {
        visitor.visitTouchSensor(this);
    }

    @Override
    public ISessionData resetSession()
    {
        mValue = 0;
        return this;
    }
}